@@ -1,12 +1,12 @@
 use crate::auditwheel::{AuditWheelMode, PlatformTag};
-use crate::build_context::BridgeModel;
 use crate::compile::{CompileTarget, LIB_CRATE_TYPES};
 use crate::cross_compile::{find_sysconfigdata, parse_sysconfigdata};
 use crate::project_layout::ProjectResolver;
 use crate::pyproject_toml::ToolMaturin;
-use crate::python_interpreter::{InterpreterConfig, InterpreterKind, MINIMUM_PYTHON_MINOR};
-use crate::{BuildContext, PythonInterpreter, Target};
+use crate::python_interpreter::{InterpreterConfig, InterpreterKind};
+use crate::{Bindings, BridgeModel, BuildContext, PythonInterpreter, Target};
 use anyhow::{bail, format_err, Context, Result};
+use cargo_metadata::{CrateType, PackageId, TargetKind};
 use cargo_metadata::{Metadata, Node};
 use cargo_options::heading;
 use pep440_rs::VersionSpecifiers;
@@ -23,22 +23,6 @@ use tracing::{debug, instrument};
 // and more restrictive.
 const PYO3_BINDING_CRATES: [&str; 2] = ["pyo3-ffi", "pyo3"];
 
-fn pyo3_minimum_python_minor_version(major_version: u64, minor_version: u64) -> Option<usize> {
-    if (major_version, minor_version) >= (0, 16) {
-        Some(7)
-    } else {
-        None
-    }
-}
-
-fn pyo3_ffi_minimum_python_minor_version(major_version: u64, minor_version: u64) -> Option<usize> {
-    if (major_version, minor_version) >= (0, 16) {
-        pyo3_minimum_python_minor_version(major_version, minor_version)
-    } else {
-        None
-    }
-}
-
 /// Cargo options for the build process
 #[derive(Debug, Default, Serialize, Deserialize, clap::Parser, Clone, Eq, PartialEq)]
 #[serde(default, rename_all = "kebab-case")]
@@ -232,7 +216,12 @@ impl BuildOptions {
         generate_import_lib: bool,
     ) -> Result<Vec<PythonInterpreter>> {
         match bridge {
-            BridgeModel::Bindings(binding_name, _) | BridgeModel::Bin(Some((binding_name, _))) => {
+            BridgeModel::Bindings(Bindings {
+                name: binding_name, ..
+            })
+            | BridgeModel::Bin(Some(Bindings {
+                name: binding_name, ..
+            })) => {
                 let mut interpreters = Vec::new();
                 if let Some(config_file) = env::var_os("PYO3_CONFIG_FILE") {
                     if !binding_name.starts_with("pyo3") {
@@ -274,6 +263,10 @@ impl BuildOptions {
                             .get("ABIFLAGS")
                             .map(ToString::to_string)
                             .unwrap_or_default();
+                        let gil_disabled = sysconfig_data
+                            .get("Py_GIL_DISABLED")
+                            .map(|x| x == "1")
+                            .unwrap_or_default();
                         let ext_suffix = sysconfig_data
                             .get("EXT_SUFFIX")
                             .context("syconfig didn't define an `EXT_SUFFIX` ಠ_ಠ")?;
@@ -299,6 +292,7 @@ impl BuildOptions {
                                 abiflags,
                                 ext_suffix: ext_suffix.to_string(),
                                 pointer_width: None,
+                                gil_disabled,
                             },
                             executable: PathBuf::new(),
                             platform: None,
@@ -319,8 +313,12 @@ impl BuildOptions {
                                 bail!("{} is not a valid python interpreter", interp.display());
                             }
                         }
-                        interpreters =
-                            find_interpreter_in_sysconfig(interpreter, target, requires_python)?;
+                        interpreters = find_interpreter_in_sysconfig(
+                            bridge,
+                            interpreter,
+                            target,
+                            requires_python,
+                        )?;
                         if interpreters.is_empty() {
                             bail!(
                                 "Couldn't find any python interpreters from '{}'. Please check that both major and minor python version have been specified in -i/--interpreter.",
@@ -356,14 +354,29 @@ impl BuildOptions {
                 Ok(vec![interpreter])
             }
             BridgeModel::Bin(None) | BridgeModel::UniFfi => Ok(vec![]),
-            BridgeModel::BindingsAbi3(major, minor) => {
+            BridgeModel::BindingsAbi3 { major, minor, .. } => {
+                let found_interpreters =
+                    find_interpreter_in_host(bridge, interpreter, target, requires_python)
+                        .or_else(|err| {
+                            // Can only use sysconfig-derived interpreter on windows if generating the import lib
+                            if target.is_windows() && !generate_import_lib {
+                                return Err(err.context("Need a Python interpreter to compile for Windows without PyO3's `generate-import-lib` feature"));
+                            }
+
+                            let interps =
+                                find_interpreter_in_sysconfig(bridge,interpreter, target, requires_python)
+                                    .unwrap_or_default();
+                            if interps.is_empty() && !self.interpreter.is_empty() {
+                                // Print error when user supplied `--interpreter` option
+                                Err(err)
+                            } else {
+                                Ok(interps)
+                            }
+                        })?;
+
                 if target.is_windows() {
-                    // Ideally, we wouldn't want to use any python interpreter without abi3 at all.
-                    // Unfortunately, on windows we need one to figure out base_prefix for a linker
-                    // argument.
-                    let interpreters =
-                        find_interpreter_in_host(bridge, interpreter, target, requires_python)
-                            .unwrap_or_default();
+                    // On windows we might need a Python executable to locate a base prefix for
+                    // linker args, if we're not using PyO3's `generate-import-lib` feature.
                     if env::var_os("PYO3_CROSS_LIB_DIR").is_some() {
                         // PYO3_CROSS_LIB_DIR should point to the `libs` directory inside base_prefix
                         // when cross compiling, so we fake a python interpreter matching it
@@ -376,6 +389,7 @@ impl BuildOptions {
                                 abiflags: "".to_string(),
                                 ext_suffix: ".pyd".to_string(),
                                 pointer_width: None,
+                                gil_disabled: false,
                             },
                             executable: PathBuf::new(),
                             platform: None,
@@ -388,47 +402,36 @@ impl BuildOptions {
                             InterpreterConfig::from_pyo3_config(config_file.as_ref(), target)
                                 .context("Invalid PYO3_CONFIG_FILE")?;
                         Ok(vec![PythonInterpreter::from_config(interpreter_config)])
-                    } else if let Some(interp) = interpreters.first() {
-                        eprintln!("🐍 Using {interp} to generate to link bindings (With abi3, an interpreter is only required on windows)");
-                        Ok(interpreters)
                     } else if generate_import_lib {
-                        eprintln!("🐍 Not using a specific python interpreter (Automatically generating windows import library)");
-                        // fake a python interpreter
-                        Ok(vec![PythonInterpreter {
-                            config: InterpreterConfig {
-                                major: *major as usize,
-                                minor: *minor as usize,
-                                interpreter_kind: InterpreterKind::CPython,
-                                abiflags: "".to_string(),
-                                ext_suffix: ".pyd".to_string(),
-                                pointer_width: None,
-                            },
-                            executable: PathBuf::new(),
-                            platform: None,
-                            runnable: false,
-                            implementation_name: "cpython".to_string(),
-                            soabi: None,
-                        }])
+                        eprintln!("🐍 Not using a specific python interpreter (automatically generating windows import library)");
+                        let mut found_interpreters = found_interpreters;
+                        // fake a python interpreter if none directly found
+                        if found_interpreters.is_empty() {
+                            found_interpreters.push(PythonInterpreter {
+                                config: InterpreterConfig {
+                                    major: *major as usize,
+                                    minor: *minor as usize,
+                                    interpreter_kind: InterpreterKind::CPython,
+                                    abiflags: "".to_string(),
+                                    ext_suffix: ".pyd".to_string(),
+                                    pointer_width: None,
+                                    gil_disabled: false,
+                                },
+                                executable: PathBuf::new(),
+                                platform: None,
+                                runnable: false,
+                                implementation_name: "cpython".to_string(),
+                                soabi: None,
+                            })
+                        }
+                        Ok(found_interpreters)
                     } else {
-                        bail!("Failed to find a python interpreter");
+                        if found_interpreters.is_empty() {
+                            bail!("Failed to find any python interpreter");
+                        }
+                        Ok(found_interpreters)
                     }
                 } else {
-                    let found_interpreters =
-                        find_interpreter_in_host(bridge, interpreter, target, requires_python)
-                            .or_else(|err| {
-                                let interps = find_interpreter_in_sysconfig(
-                                    interpreter,
-                                    target,
-                                    requires_python,
-                                )
-                                .unwrap_or_default();
-                                if interps.is_empty() && !self.interpreter.is_empty() {
-                                    // Print error when user supplied `--interpreter` option
-                                    Err(err)
-                                } else {
-                                    Ok(interps)
-                                }
-                            })?;
                     eprintln!("🐍 Not using a specific python interpreter");
                     if self.interpreter.is_empty() {
                         // Fake one to make `BuildContext::build_wheels` happy for abi3 when no cpython/pypy found on host
@@ -441,6 +444,7 @@ impl BuildOptions {
                                 abiflags: "".to_string(),
                                 ext_suffix: "".to_string(),
                                 pointer_width: None,
+                                gil_disabled: false,
                             },
                             executable: PathBuf::new(),
                             platform: None,
@@ -465,6 +469,7 @@ impl BuildOptions {
                         // we cannot use host pypy so switch to bundled sysconfig instead
                         if !pypys.is_empty() {
                             interps.extend(find_interpreter_in_sysconfig(
+                                bridge,
                                 &pypys,
                                 target,
                                 requires_python,
@@ -487,12 +492,59 @@ impl BuildOptions {
 
     /// Tries to fill the missing metadata for a BuildContext by querying cargo and python
     #[instrument(skip_all)]
-    pub fn into_build_context(
-        self,
-        release: bool,
-        strip: bool,
-        editable: bool,
-    ) -> Result<BuildContext> {
+    pub fn into_build_context(self) -> BuildContextBuilder {
+        BuildContextBuilder::new(self)
+    }
+}
+
+#[derive(Debug)]
+pub struct BuildContextBuilder {
+    build_options: BuildOptions,
+    release: bool,
+    strip: bool,
+    editable: bool,
+    sdist_only: bool,
+}
+
+impl BuildContextBuilder {
+    fn new(build_options: BuildOptions) -> Self {
+        Self {
+            build_options,
+            release: false,
+            strip: false,
+            editable: false,
+            sdist_only: false,
+        }
+    }
+
+    pub fn release(mut self, release: bool) -> Self {
+        self.release = release;
+        self
+    }
+
+    pub fn strip(mut self, strip: bool) -> Self {
+        self.strip = strip;
+        self
+    }
+
+    pub fn editable(mut self, editable: bool) -> Self {
+        self.editable = editable;
+        self
+    }
+
+    pub fn sdist_only(mut self, sdist_only: bool) -> Self {
+        self.sdist_only = sdist_only;
+        self
+    }
+
+    pub fn build(self) -> Result<BuildContext> {
+        let Self {
+            build_options,
+            release,
+            strip,
+            editable,
+            sdist_only,
+        } = self;
         let ProjectResolver {
             project_layout,
             cargo_toml_path,
@@ -500,16 +552,19 @@ impl BuildOptions {
             pyproject_toml_path,
             pyproject_toml,
             module_name,
-            metadata23,
+            metadata24,
             mut cargo_options,
             cargo_metadata,
             mut pyproject_toml_maturin_options,
-        } = ProjectResolver::resolve(self.manifest_path.clone(), self.cargo.clone())?;
+        } = ProjectResolver::resolve(
+            build_options.manifest_path.clone(),
+            build_options.cargo.clone(),
+        )?;
         let pyproject = pyproject_toml.as_ref();
 
         let bridge = find_bridge(
             &cargo_metadata,
-            self.bindings.as_deref().or_else(|| {
+            build_options.bindings.as_deref().or_else(|| {
                 pyproject.and_then(|x| {
                     if x.bindings().is_some() {
                         pyproject_toml_maturin_options.push("bindings");
@@ -527,7 +582,7 @@ impl BuildOptions {
             );
         }
 
-        let mut target_triple = self.target.clone();
+        let mut target_triple = build_options.target.clone();
 
         let mut universal2 = target_triple.as_deref() == Some("universal2-apple-darwin");
         // Also try to determine universal2 from ARCHFLAGS environment variable
@@ -560,7 +615,7 @@ impl BuildOptions {
 
         let mut target = Target::from_target_triple(target_triple)?;
         if !target.user_specified && !universal2 {
-            if let Some(interpreter) = self.interpreter.first() {
+            if let Some(interpreter) = build_options.interpreter.first() {
                 if let Some(detected_target) =
                     crate::target::detect_arch_from_python(interpreter, &target)
                 {
@@ -569,38 +624,23 @@ impl BuildOptions {
             }
         }
 
-        let wheel_dir = match self.out {
+        let wheel_dir = match build_options.out {
             Some(ref dir) => dir.clone(),
             None => PathBuf::from(&cargo_metadata.target_directory).join("wheels"),
         };
 
         let generate_import_lib = is_generating_import_lib(&cargo_metadata)?;
-        let interpreter = if self.find_interpreter {
-            // Auto-detect interpreters
-            self.find_interpreters(
+        let interpreter = if sdist_only && env::var_os("MATURIN_TEST_PYTHON").is_none() {
+            // We don't need a python interpreter to build sdist only
+            Vec::new()
+        } else {
+            resolve_interpreters(
+                &build_options,
                 &bridge,
-                &[],
                 &target,
-                metadata23.requires_python.as_ref(),
+                metadata24.requires_python.as_ref(),
                 generate_import_lib,
             )?
-        } else {
-            // User given list of interpreters
-            let interpreter = if self.interpreter.is_empty() && !target.cross_compiling() {
-                if cfg!(test) {
-                    match env::var_os("MATURIN_TEST_PYTHON") {
-                        Some(python) => vec![python.into()],
-                        None => vec![target.get_python()],
-                    }
-                } else {
-                    vec![target.get_python()]
-                }
-            } else {
-                // XXX: False positive clippy warning
-                #[allow(clippy::redundant_clone)]
-                self.interpreter.clone()
-            };
-            self.find_interpreters(&bridge, &interpreter, &target, None, generate_import_lib)?
         };
 
         if cargo_options.args.is_empty() {
@@ -613,9 +653,9 @@ impl BuildOptions {
         }
 
         let strip = pyproject.map(|x| x.strip()).unwrap_or_default() || strip;
-        let skip_auditwheel =
-            pyproject.map(|x| x.skip_auditwheel()).unwrap_or_default() || self.skip_auditwheel;
-        let auditwheel = self
+        let skip_auditwheel = pyproject.map(|x| x.skip_auditwheel()).unwrap_or_default()
+            || build_options.skip_auditwheel;
+        let auditwheel = build_options
             .auditwheel
             .or_else(|| pyproject.and_then(|x| x.auditwheel()))
             .unwrap_or(if skip_auditwheel {
@@ -623,9 +663,9 @@ impl BuildOptions {
             } else {
                 AuditWheelMode::Repair
             });
-        let platform_tags = if self.platform_tag.is_empty() {
+        let platform_tags = if build_options.platform_tag.is_empty() {
             #[cfg(feature = "zig")]
-            let use_zig = self.zig;
+            let use_zig = build_options.zig;
             #[cfg(not(feature = "zig"))]
             let use_zig = false;
             let compatibility = pyproject
@@ -658,7 +698,7 @@ impl BuildOptions {
                 Vec::new()
             }
         } else {
-            self.platform_tag
+            build_options.platform_tag
         };
 
         for platform_tag in &platform_tags {
@@ -683,7 +723,7 @@ impl BuildOptions {
             );
         }
 
-        let target_dir = self
+        let target_dir = build_options
             .cargo
             .target_dir
             .clone()
@@ -703,7 +743,7 @@ impl BuildOptions {
             project_layout,
             pyproject_toml_path,
             pyproject_toml,
-            metadata23,
+            metadata24,
             crate_name,
             module_name,
             manifest_path: cargo_toml_path,
@@ -713,7 +753,7 @@ impl BuildOptions {
             strip,
             auditwheel,
             #[cfg(feature = "zig")]
-            zig: self.zig,
+            zig: build_options.zig,
             platform_tag: platform_tags,
             interpreter,
             cargo_metadata,
@@ -724,6 +764,43 @@ impl BuildOptions {
     }
 }
 
+fn resolve_interpreters(
+    build_options: &BuildOptions,
+    bridge: &BridgeModel,
+    target: &Target,
+    requires_python: Option<&VersionSpecifiers>,
+    generate_import_lib: bool,
+) -> Result<Vec<PythonInterpreter>, anyhow::Error> {
+    let interpreter = if build_options.find_interpreter {
+        // Auto-detect interpreters
+        build_options.find_interpreters(
+            bridge,
+            &[],
+            target,
+            requires_python,
+            generate_import_lib,
+        )?
+    } else {
+        // User given list of interpreters
+        let interpreter = if build_options.interpreter.is_empty() && !target.cross_compiling() {
+            if cfg!(test) {
+                match env::var_os("MATURIN_TEST_PYTHON") {
+                    Some(python) => vec![python.into()],
+                    None => vec![target.get_python()],
+                }
+            } else {
+                vec![target.get_python()]
+            }
+        } else {
+            // XXX: False positive clippy warning
+            #[allow(clippy::redundant_clone)]
+            build_options.interpreter.clone()
+        };
+        build_options.find_interpreters(bridge, &interpreter, target, None, generate_import_lib)?
+    };
+    Ok(interpreter)
+}
+
 /// Checks for bridge/platform type edge cases
 fn validate_bridge_type(
     bridge: &BridgeModel,
@@ -801,7 +878,7 @@ fn filter_cargo_targets(
                             .all(|f| resolved_features.contains(f))
                 }
             }
-            _ => target.kind.contains(&"cdylib".to_string()),
+            _ => target.crate_types.contains(&CrateType::CDyLib),
         })
         .map(|target| CompileTarget {
             target: target.clone(),
@@ -813,9 +890,9 @@ fn filter_cargo_targets(
         // Let's try compile one of the target with `--crate-type cdylib`
         let lib_target = root_pkg.targets.iter().find(|target| {
             target
-                .kind
+                .crate_types
                 .iter()
-                .any(|k| LIB_CRATE_TYPES.contains(&k.as_str()))
+                .any(|crate_type| LIB_CRATE_TYPES.contains(crate_type))
         });
         if let Some(target) = lib_target {
             targets.push(CompileTarget {
@@ -831,7 +908,7 @@ fn filter_cargo_targets(
             config_targets.iter().any(|config_target| {
                 let name_eq = config_target.name == target.name;
                 match &config_target.kind {
-                    Some(kind) => name_eq && target.kind.contains(kind),
+                    Some(kind) => name_eq && target.crate_types.contains(&CrateType::from(*kind)),
                     None => name_eq,
                 }
             })
@@ -857,46 +934,34 @@ fn filter_cargo_targets(
 }
 
 /// pyo3 supports building abi3 wheels if the unstable-api feature is not selected
-fn has_abi3(cargo_metadata: &Metadata) -> Result<Option<(u8, u8)>> {
-    let resolve = cargo_metadata
-        .resolve
-        .as_ref()
-        .context("Expected cargo to return metadata with resolve")?;
+fn has_abi3(deps: &HashMap<&str, &Node>) -> Result<Option<(u8, u8)>> {
     for &lib in PYO3_BINDING_CRATES.iter() {
-        let pyo3_packages = resolve
-            .nodes
-            .iter()
-            .filter(|package| cargo_metadata[&package.id].name.as_str() == lib)
-            .collect::<Vec<_>>();
-        match pyo3_packages.as_slice() {
-            [pyo3_crate] => {
-                // Find the minimal abi3 python version. If there is none, abi3 hasn't been selected
-                // This parser abi3-py{major}{minor} and returns the minimal (major, minor) tuple
-                let abi3_selected = pyo3_crate.features.iter().any(|x| x == "abi3");
+        if let Some(pyo3_crate) = deps.get(lib) {
+            // Find the minimal abi3 python version. If there is none, abi3 hasn't been selected
+            // This parser abi3-py{major}{minor} and returns the minimal (major, minor) tuple
+            let abi3_selected = pyo3_crate.features.iter().any(|x| x == "abi3");
 
-                let min_abi3_version = pyo3_crate
-                    .features
-                    .iter()
-                    .filter(|x| x.starts_with("abi3-py") && x.len() >= "abi3-pyxx".len())
-                    .map(|x| {
-                        Ok((
-                            (x.as_bytes()[7] as char).to_string().parse::<u8>()?,
-                            x[8..].parse::<u8>()?,
-                        ))
-                    })
-                    .collect::<Result<Vec<(u8, u8)>>>()
-                    .context(format!("Bogus {lib} cargo features"))?
-                    .into_iter()
-                    .min();
-                if abi3_selected && min_abi3_version.is_none() {
-                    bail!(
+            let min_abi3_version = pyo3_crate
+                .features
+                .iter()
+                .filter(|x| x.starts_with("abi3-py") && x.len() >= "abi3-pyxx".len())
+                .map(|x| {
+                    Ok((
+                        (x.as_bytes()[7] as char).to_string().parse::<u8>()?,
+                        x[8..].parse::<u8>()?,
+                    ))
+                })
+                .collect::<Result<Vec<(u8, u8)>>>()
+                .context(format!("Bogus {lib} cargo features"))?
+                .into_iter()
+                .min();
+            if abi3_selected && min_abi3_version.is_none() {
+                bail!(
                         "You have selected the `abi3` feature but not a minimum version (e.g. the `abi3-py36` feature). \
                         maturin needs a minimum version feature to build abi3 wheels."
                     )
-                }
-                return Ok(min_abi3_version);
             }
-            _ => continue,
+            return Ok(min_abi3_version);
         }
     }
     Ok(None)
@@ -933,36 +998,71 @@ fn is_generating_import_lib(cargo_metadata: &Metadata) -> Result<bool> {
 fn find_bindings(
     deps: &HashMap<&str, &Node>,
     packages: &HashMap<&str, &cargo_metadata::Package>,
-) -> Option<(String, usize)> {
+) -> Option<Bindings> {
     if deps.get("pyo3").is_some() {
-        let ver = &packages["pyo3"].version;
-        let minor =
-            pyo3_minimum_python_minor_version(ver.major, ver.minor).unwrap_or(MINIMUM_PYTHON_MINOR);
-        Some(("pyo3".to_string(), minor))
+        let version = packages["pyo3"].version.clone();
+        Some(Bindings {
+            name: "pyo3".to_string(),
+            version,
+        })
     } else if deps.get("pyo3-ffi").is_some() {
-        let ver = &packages["pyo3-ffi"].version;
-        let minor = pyo3_ffi_minimum_python_minor_version(ver.major, ver.minor)
-            .unwrap_or(MINIMUM_PYTHON_MINOR);
-        Some(("pyo3-ffi".to_string(), minor))
+        let version = packages["pyo3-ffi"].version.clone();
+        Some(Bindings {
+            name: "pyo3-ffi".to_string(),
+            version,
+        })
     } else if deps.contains_key("uniffi") {
-        Some(("uniffi".to_string(), MINIMUM_PYTHON_MINOR))
+        let version = packages["uniffi"].version.clone();
+        Some(Bindings {
+            name: "uniffi".to_string(),
+            version,
+        })
     } else {
         None
     }
 }
 
-/// Tries to determine the [BridgeModel] for the target crate
-pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<BridgeModel> {
+/// Return a map with all (transitive) dependencies of the *current* crate.
+/// This is different from `metadata.resolve`, which also includes packages
+/// that are used in the same workspace, but on which the current crate does not depend.
+fn current_crate_dependencies(cargo_metadata: &Metadata) -> Result<HashMap<&str, &Node>> {
     let resolve = cargo_metadata
         .resolve
         .as_ref()
-        .ok_or_else(|| format_err!("Expected to get a dependency graph from cargo"))?;
+        .context("Expected to get a dependency graph from cargo")?;
+    let root = resolve
+        .root
+        .as_ref()
+        .context("expected to get a root package")?;
+    let nodes: HashMap<&PackageId, &Node> =
+        resolve.nodes.iter().map(|node| (&node.id, node)).collect();
 
-    let deps: HashMap<&str, &Node> = resolve
-        .nodes
-        .iter()
-        .map(|node| (cargo_metadata[&node.id].name.as_ref(), node))
-        .collect();
+    // Walk the dependency tree to get all (in)direct children.
+    let mut dep_ids = HashSet::with_capacity(nodes.len());
+    let mut todo = Vec::from([root]);
+    while let Some(id) = todo.pop() {
+        for dep in nodes[id].deps.iter() {
+            if dep_ids.contains(&dep.pkg) {
+                continue;
+            }
+            dep_ids.insert(&dep.pkg);
+            todo.push(&dep.pkg);
+        }
+    }
+
+    Ok(nodes
+        .into_iter()
+        .filter_map(|(id, node)| {
+            dep_ids
+                .contains(&id)
+                .then_some((cargo_metadata[id].name.as_ref(), node))
+        })
+        .collect())
+}
+
+/// Tries to determine the [BridgeModel] for the target crate
+pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<BridgeModel> {
+    let deps = current_crate_dependencies(cargo_metadata)?;
     let packages: HashMap<&str, &cargo_metadata::Package> = cargo_metadata
         .packages
         .iter()
@@ -983,11 +1083,17 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
         .iter()
         .filter(|target| {
             target.kind.iter().any(|kind| {
-                kind != "example" && kind != "test" && kind != "bench" && kind != "custom-build"
+                !matches!(
+                    kind,
+                    TargetKind::Bench
+                        | TargetKind::CustomBuild
+                        | TargetKind::Example
+                        | TargetKind::ProcMacro
+                        | TargetKind::Test
+                )
             })
         })
-        .flat_map(|target| target.crate_types.iter())
-        .map(String::as_str)
+        .flat_map(|target| target.crate_types.iter().cloned())
         .collect();
 
     let bridge = if let Some(bindings) = bridge {
@@ -998,7 +1104,7 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
         } else if bindings == "bin" {
             // uniffi bindings don't support bin
             let bindings =
-                find_bindings(&deps, &packages).filter(|(bindings, _)| bindings != "uniffi");
+                find_bindings(&deps, &packages).filter(|bindings| bindings.name != "uniffi");
             BridgeModel::Bin(bindings)
         } else {
             if !deps.contains_key(bindings) {
@@ -1008,24 +1114,28 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
                 );
             }
 
-            BridgeModel::Bindings(bindings.to_string(), MINIMUM_PYTHON_MINOR)
+            let version = packages[bindings].version.clone();
+            BridgeModel::Bindings(Bindings {
+                name: bindings.to_string(),
+                version,
+            })
         }
-    } else if let Some((bindings, minor)) = find_bindings(&deps, &packages) {
-        if !targets.contains(&"cdylib") && targets.contains(&"bin") {
-            if bindings == "uniffi" {
+    } else if let Some(bindings) = find_bindings(&deps, &packages) {
+        if !targets.contains(&CrateType::CDyLib) && targets.contains(&CrateType::Bin) {
+            if bindings.name == "uniffi" {
                 // uniffi bindings don't support bin
                 BridgeModel::Bin(None)
             } else {
-                BridgeModel::Bin(Some((bindings, minor)))
+                BridgeModel::Bin(Some(bindings))
             }
-        } else if bindings == "uniffi" {
+        } else if bindings.name == "uniffi" {
             BridgeModel::UniFfi
         } else {
-            BridgeModel::Bindings(bindings, minor)
+            BridgeModel::Bindings(bindings)
         }
-    } else if targets.contains(&"cdylib") {
+    } else if targets.contains(&CrateType::CDyLib) {
         BridgeModel::Cffi
-    } else if targets.contains(&"bin") {
+    } else if targets.contains(&CrateType::Bin) {
         BridgeModel::Bin(find_bindings(&deps, &packages))
     } else {
         bail!("Couldn't detect the binding type; Please specify them with --bindings/-b")
@@ -1044,13 +1154,22 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
                 eprintln!(
                     "⚠️  Warning: You're building a library without activating {lib}'s \
                      `extension-module` feature. \
-                     See https://pyo3.rs/v{version}/building_and_distribution.html#linking"
+                     See https://pyo3.rs/v{version}/building-and-distribution.html#the-extension-module-feature"
                 );
             }
 
-            return if let Some((major, minor)) = has_abi3(cargo_metadata)? {
+            return if let Some((major, minor)) = has_abi3(&deps)? {
                 eprintln!("🔗 Found {lib} bindings with abi3 support for Python ≥ {major}.{minor}");
-                Ok(BridgeModel::BindingsAbi3(major, minor))
+                let version = packages[lib].version.clone();
+                let bindings = Bindings {
+                    name: lib.to_string(),
+                    version,
+                };
+                Ok(BridgeModel::BindingsAbi3 {
+                    bindings,
+                    major,
+                    minor,
+                })
             } else {
                 eprintln!("🔗 Found {lib} bindings");
                 Ok(bridge)
@@ -1106,7 +1225,7 @@ fn find_interpreter(
         }
         if !missing.is_empty() {
             let sysconfig_interps =
-                find_interpreter_in_sysconfig(&missing, target, requires_python)?;
+                find_interpreter_in_sysconfig(bridge, &missing, target, requires_python)?;
             found_interpreters.extend(sysconfig_interps);
         }
     } else {
@@ -1162,28 +1281,37 @@ fn find_interpreter_in_host(
 
 /// Find python interpreters in the bundled sysconfig
 fn find_interpreter_in_sysconfig(
+    bridge: &BridgeModel,
     interpreter: &[PathBuf],
     target: &Target,
     requires_python: Option<&VersionSpecifiers>,
 ) -> Result<Vec<PythonInterpreter>> {
     if interpreter.is_empty() {
-        return Ok(PythonInterpreter::find_by_target(target, requires_python));
+        return Ok(PythonInterpreter::find_by_target(
+            target,
+            requires_python,
+            Some(bridge),
+        ));
     }
     let mut interpreters = Vec::new();
     for interp in interpreter {
         let python = interp.display().to_string();
-        let (python_impl, python_ver) = if let Some(ver) = python.strip_prefix("pypy") {
-            (InterpreterKind::PyPy, ver.strip_prefix('-').unwrap_or(ver))
+        let (python_impl, python_ver, abiflags) = if let Some(ver) = python.strip_prefix("pypy") {
+            (
+                InterpreterKind::PyPy,
+                ver.strip_prefix('-').unwrap_or(ver),
+                "",
+            )
         } else if let Some(ver) = python.strip_prefix("graalpy") {
             (
                 InterpreterKind::GraalPy,
                 ver.strip_prefix('-').unwrap_or(ver),
+                "",
             )
         } else if let Some(ver) = python.strip_prefix("python") {
-            (
-                InterpreterKind::CPython,
-                ver.strip_prefix('-').unwrap_or(ver),
-            )
+            // Also accept things like `python3.13t` for free-threaded python
+            let (ver, abiflags) = maybe_free_threaded(ver.strip_prefix('-').unwrap_or(ver));
+            (InterpreterKind::CPython, ver, abiflags)
         } else if python
             .chars()
             .next()
@@ -1191,7 +1319,8 @@ fn find_interpreter_in_sysconfig(
             .unwrap_or(false)
         {
             // Eg: -i 3.9 without interpreter kind, assume it's CPython
-            (InterpreterKind::CPython, &*python)
+            let (ver, abiflags) = maybe_free_threaded(&python);
+            (InterpreterKind::CPython, ver, abiflags)
         } else {
             // if interpreter not known
             if std::path::Path::new(&python).is_file() {
@@ -1212,7 +1341,12 @@ fn find_interpreter_in_sysconfig(
         let ver_minor = ver_minor.parse::<usize>().with_context(|| {
             format!("Invalid python interpreter minor version '{ver_minor}', expect a digit")
         })?;
-        let sysconfig = InterpreterConfig::lookup_one(target, python_impl, (ver_major, ver_minor))
+
+        if (ver_major, ver_minor) < (3, 13) && abiflags == "t" {
+            bail!("Free-threaded Python interpreter is only supported on 3.13 and later.");
+        }
+
+        let sysconfig = InterpreterConfig::lookup_one(target, python_impl, (ver_major, ver_minor), abiflags)
             .with_context(|| {
                 format!("Failed to find a {python_impl} {ver_major}.{ver_minor} interpreter in known sysconfig")
             })?;
@@ -1225,6 +1359,14 @@ fn find_interpreter_in_sysconfig(
     Ok(interpreters)
 }
 
+fn maybe_free_threaded(python_ver: &str) -> (&str, &str) {
+    if let Some(ver) = python_ver.strip_suffix('t') {
+        (ver, "t")
+    } else {
+        (python_ver, "")
+    }
+}
+
 /// We need to pass the global flags to cargo metadata
 /// (https://github.com/PyO3/maturin/issues/211 and https://github.com/PyO3/maturin/issues/472),
 /// but we can't pass all the extra args, as e.g. `--target` isn't supported, so this tries to
@@ -1381,11 +1523,11 @@ mod test {
 
         assert!(matches!(
             find_bridge(&pyo3_mixed, None),
-            Ok(BridgeModel::Bindings(..))
+            Ok(BridgeModel::Bindings { .. })
         ));
         assert!(matches!(
             find_bridge(&pyo3_mixed, Some("pyo3")),
-            Ok(BridgeModel::Bindings(..))
+            Ok(BridgeModel::Bindings { .. })
         ));
     }
 
@@ -1396,14 +1538,16 @@ mod test {
             .exec()
             .unwrap();
 
-        assert!(matches!(
-            find_bridge(&pyo3_pure, None),
-            Ok(BridgeModel::BindingsAbi3(3, 7))
-        ));
-        assert!(matches!(
-            find_bridge(&pyo3_pure, Some("pyo3")),
-            Ok(BridgeModel::BindingsAbi3(3, 7))
-        ));
+        let bridge = BridgeModel::BindingsAbi3 {
+            bindings: Bindings {
+                name: "pyo3".to_string(),
+                version: semver::Version::new(0, 24, 0),
+            },
+            major: 3,
+            minor: 7,
+        };
+        assert_eq!(find_bridge(&pyo3_pure, None).unwrap(), bridge);
+        assert_eq!(find_bridge(&pyo3_pure, Some("pyo3")).unwrap(), bridge);
     }
 
     #[test]
@@ -1423,7 +1567,7 @@ mod test {
 
         assert!(matches!(
             find_bridge(&pyo3_pure, None).unwrap(),
-            BridgeModel::Bindings(..)
+            BridgeModel::Bindings { .. }
         ));
     }
 
@@ -1467,11 +1611,11 @@ mod test {
             .unwrap();
         assert!(matches!(
             find_bridge(&pyo3_bin, Some("bin")).unwrap(),
-            BridgeModel::Bin(Some((..)))
+            BridgeModel::Bin(Some(_))
         ));
         assert!(matches!(
             find_bridge(&pyo3_bin, None).unwrap(),
-            BridgeModel::Bin(Some(..))
+            BridgeModel::Bin(Some(_))
         ));
     }
 
