@@ -273,14 +273,19 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             strip,
         } => {
             assert_eq!(build_options.interpreter.len(), 1);
-            let context = build_options.into_build_context(true, strip, false)?;
+            let context = build_options
+                .into_build_context()
+                .release(true)
+                .strip(strip)
+                .editable(false)
+                .build()?;
 
             // Since afaik all other PEP 517 backends also return linux tagged wheels, we do so too
             let tags = match context.bridge() {
                 BridgeModel::Bindings(..) | BridgeModel::Bin(Some(..)) => {
                     vec![context.interpreter[0].get_tag(&context, &[PlatformTag::Linux])?]
                 }
-                BridgeModel::BindingsAbi3(major, minor) => {
+                BridgeModel::BindingsAbi3 { major, minor, .. } => {
                     let platform = context.get_platform_tag(&[PlatformTag::Linux])?;
                     vec![format!("cp{major}{minor}-abi3-{platform}")]
                 }
@@ -290,15 +295,20 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             };
 
             let mut writer = PathWriter::from_path(metadata_directory);
-            write_dist_info(&mut writer, &context.metadata23, &tags)?;
-            println!("{}", context.metadata23.get_dist_info_dir().display());
+            write_dist_info(&mut writer, &context.metadata24, &tags)?;
+            println!("{}", context.metadata24.get_dist_info_dir().display());
         }
         Pep517Command::BuildWheel {
             build_options,
             strip,
             editable,
         } => {
-            let build_context = build_options.into_build_context(true, strip, editable)?;
+            let build_context = build_options
+                .into_build_context()
+                .release(true)
+                .strip(strip)
+                .editable(editable)
+                .build()?;
             let wheels = build_context.build_wheels()?;
             assert_eq!(wheels.len(), 1);
             println!("{}", wheels[0].0.to_str().unwrap());
@@ -318,7 +328,13 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
                 },
                 ..Default::default()
             };
-            let build_context = build_options.into_build_context(false, false, false)?;
+            let build_context = build_options
+                .into_build_context()
+                .release(false)
+                .strip(false)
+                .editable(false)
+                .sdist_only(true)
+                .build()?;
             let (path, _) = build_context
                 .build_source_distribution()?
                 .context("Failed to build source distribution, pyproject.toml not found")?;
@@ -361,7 +377,12 @@ fn run() -> Result<()> {
             strip,
             sdist,
         } => {
-            let build_context = build.into_build_context(release, strip, false)?;
+            let build_context = build
+                .into_build_context()
+                .release(release)
+                .strip(strip)
+                .editable(false)
+                .build()?;
             if sdist {
                 build_context
                     .build_source_distribution()?
@@ -378,7 +399,12 @@ fn run() -> Result<()> {
             no_strip,
             no_sdist,
         } => {
-            let build_context = build.into_build_context(!debug, !no_strip, false)?;
+            let build_context = build
+                .into_build_context()
+                .release(!debug)
+                .strip(!no_strip)
+                .editable(false)
+                .build()?;
 
             if !build_context.release {
                 eprintln!("⚠️  Warning: You're publishing debug wheels");
@@ -399,7 +425,7 @@ fn run() -> Result<()> {
         Command::ListPython { target } => {
             let found = if target.is_some() {
                 let target = Target::from_target_triple(target)?;
-                PythonInterpreter::find_by_target(&target, None)
+                PythonInterpreter::find_by_target(&target, None, None)
             } else {
                 let target = Target::from_target_triple(None)?;
                 // We don't know the targeted bindings yet, so we use the most lenient
@@ -427,7 +453,13 @@ fn run() -> Result<()> {
                 },
                 ..Default::default()
             };
-            let build_context = build_options.into_build_context(false, false, false)?;
+            let build_context = build_options
+                .into_build_context()
+                .release(false)
+                .strip(false)
+                .editable(false)
+                .sdist_only(true)
+                .build()?;
             build_context
                 .build_source_distribution()?
                 .context("Failed to build source distribution, pyproject.toml not found")?;
