@@ -3,11 +3,13 @@ use anyhow::{bail, format_err, Context, Result};
 use fs_err as fs;
 use indexmap::IndexMap;
 use pep440_rs::{Version, VersionSpecifiers};
-use pep508_rs::{MarkerExpression, MarkerOperator, MarkerTree, MarkerValue, Requirement};
+use pep508_rs::{
+    ExtraName, ExtraOperator, MarkerExpression, MarkerTree, MarkerValueExtra, Requirement,
+};
 use pyproject_toml::License;
 use regex::Regex;
 use serde::{Deserialize, Serialize};
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 use std::fmt::Write as _;
 use std::path::{Path, PathBuf};
 use std::str;
@@ -16,23 +18,23 @@ use std::str::FromStr;
 /// The metadata required to generate the .dist-info directory
 #[derive(Serialize, Deserialize, Debug, Clone, Eq, PartialEq)]
 pub struct WheelMetadata {
-    /// Python Package Metadata 2.3
-    pub metadata23: Metadata23,
+    /// Python Package Metadata 2.4
+    pub metadata24: Metadata24,
     /// The `[console_scripts]` for the entry_points.txt
     pub scripts: HashMap<String, String>,
     /// The name of the module can be distinct from the package name, mostly
     /// because package names normally contain minuses while module names
-    /// have underscores. The package name is part of metadata23
+    /// have underscores. The package name is part of metadata24
     pub module_name: String,
 }
 
-/// Python Package Metadata 2.3 as specified in
+/// Python Package Metadata 2.4 as specified in
 /// https://packaging.python.org/specifications/core-metadata/
 /// Maturin writes static metadata and does not support dynamic fields atm.
 #[derive(Serialize, Deserialize, Debug, Clone, Eq, PartialEq)]
 #[serde(rename_all = "kebab-case")]
 #[allow(missing_docs)]
-pub struct Metadata23 {
+pub struct Metadata24 {
     // Mandatory fields
     pub metadata_version: String,
     pub name: String,
@@ -66,11 +68,11 @@ pub struct Metadata23 {
     pub entry_points: IndexMap<String, IndexMap<String, String>>,
 }
 
-impl Metadata23 {
+impl Metadata24 {
     /// Initializes with name, version and otherwise the defaults
     pub fn new(name: String, version: Version) -> Self {
         Self {
-            metadata_version: "2.3".to_string(),
+            metadata_version: "2.4".to_string(),
             name,
             version,
             platform: vec![],
@@ -123,7 +125,7 @@ fn path_to_content_type(path: &Path) -> String {
         })
 }
 
-impl Metadata23 {
+impl Metadata24 {
     /// Merge metadata with pyproject.toml, where pyproject.toml takes precedence
     ///
     /// pyproject_dir must be the directory containing pyproject.toml
@@ -134,8 +136,26 @@ impl Metadata23 {
     ) -> Result<()> {
         let pyproject_dir = pyproject_dir.as_ref();
         if let Some(project) = &pyproject_toml.project {
+            let dynamic: HashSet<&str> = project
+                .dynamic
+                .as_ref()
+                .map(|x| x.iter().map(AsRef::as_ref).collect())
+                .unwrap_or_default();
+            if dynamic.contains("name") {
+                bail!("`project.dynamic` must not specify `name` in pyproject.toml");
+            }
+
             self.name.clone_from(&project.name);
 
+            let version_ok = pyproject_toml.warn_invalid_version_info();
+            if !version_ok {
+                // This is a hard error for maturin>=2.0, see https://github.com/PyO3/maturin/issues/2416.
+                let current_major = env!("CARGO_PKG_VERSION_MAJOR").parse::<usize>().unwrap();
+                if current_major > 1 {
+                    bail!("Invalid version information in pyproject.toml.");
+                }
+            }
+
             if let Some(version) = &project.version {
                 self.version = version.clone();
             }
@@ -185,18 +205,12 @@ impl Metadata23 {
             if let Some(license) = &project.license {
                 match license {
                     // TODO: switch to License-Expression core metadata, see https://peps.python.org/pep-0639/#add-license-expression-field
-                    License::String(license_expr) => self.license = Some(license_expr.clone()),
-                    License::Table { file, text } => match (file, text) {
-                        (Some(_), Some(_)) => {
-                            bail!("file and text fields of 'project.license' are mutually-exclusive, only one of them should be specified");
-                        }
-                        (Some(license_path), None) => {
-                            let license_path = pyproject_dir.join(license_path);
-                            self.license_files.push(license_path);
-                        }
-                        (None, Some(license_text)) => self.license = Some(license_text.clone()),
-                        (None, None) => {}
-                    },
+                    License::Spdx(license_expr) => self.license = Some(license_expr.clone()),
+                    License::File { file } => {
+                        let license_path = pyproject_dir.join(file);
+                        self.license_files.push(license_path);
+                    }
+                    License::Text { text } => self.license = Some(text.clone()),
                 }
             }
 
@@ -228,15 +242,15 @@ impl Metadata23 {
                 let mut names = Vec::with_capacity(authors.len());
                 let mut emails = Vec::with_capacity(authors.len());
                 for author in authors {
-                    match (&author.name, &author.email) {
+                    match (author.name(), author.email()) {
                         (Some(name), Some(email)) => {
                             emails.push(escape_email_with_display_name(name, email));
                         }
                         (Some(name), None) => {
-                            names.push(name.as_str());
+                            names.push(name);
                         }
                         (None, Some(email)) => {
-                            emails.push(email.clone());
+                            emails.push(email.to_string());
                         }
                         (None, None) => {}
                     }
@@ -253,15 +267,15 @@ impl Metadata23 {
                 let mut names = Vec::with_capacity(maintainers.len());
                 let mut emails = Vec::with_capacity(maintainers.len());
                 for maintainer in maintainers {
-                    match (&maintainer.name, &maintainer.email) {
+                    match (maintainer.name(), maintainer.email()) {
                         (Some(name), Some(email)) => {
                             emails.push(escape_email_with_display_name(name, email));
                         }
                         (Some(name), None) => {
-                            names.push(name.as_str());
+                            names.push(name);
                         }
                         (None, Some(email)) => {
-                            emails.push(email.clone());
+                            emails.push(email.to_string());
                         }
                         (None, None) => {}
                     }
@@ -297,16 +311,14 @@ impl Metadata23 {
                     for dep in deps {
                         let mut dep = dep.clone();
                         // Keep in sync with `develop()`!
-                        let new_extra = MarkerTree::Expression(MarkerExpression {
-                            l_value: MarkerValue::Extra,
-                            operator: MarkerOperator::Equal,
-                            r_value: MarkerValue::QuotedString(extra.to_string()),
-                        });
-                        if let Some(existing) = dep.marker.take() {
-                            dep.marker = Some(MarkerTree::And(vec![existing, new_extra]));
-                        } else {
-                            dep.marker = Some(new_extra);
-                        }
+                        let new_extra = MarkerExpression::Extra {
+                            operator: ExtraOperator::Equal,
+                            name: MarkerValueExtra::Extra(
+                                ExtraName::new(extra.clone())
+                                    .with_context(|| format!("invalid extra name: {extra}"))?,
+                            ),
+                        };
+                        dep.marker.and(MarkerTree::expression(new_extra));
                         self.requires_dist.push(dep);
                     }
                 }
@@ -338,7 +350,7 @@ impl Metadata23 {
     pub fn from_cargo_toml(
         manifest_path: impl AsRef<Path>,
         cargo_metadata: &cargo_metadata::Metadata,
-    ) -> Result<Metadata23> {
+    ) -> Result<Metadata24> {
         let package = cargo_metadata
             .root_package()
             .context("Expected cargo to return metadata with root_package")?;
@@ -401,8 +413,8 @@ impl Metadata23 {
                 err
             )
         })?;
-        let metadata = Metadata23 {
-            // name, version and metadata_version are added through metadata23::new()
+        let metadata = Metadata24 {
+            // name, version and metadata_version are added through metadata24::new()
             // Mapped from cargo metadata
             summary: package.description.clone(),
             description,
@@ -424,7 +436,7 @@ impl Metadata23 {
             license: package.license.clone(),
             license_files,
             project_url,
-            ..Metadata23::new(name, version)
+            ..Metadata24::new(name, version)
         };
         Ok(metadata)
     }
@@ -557,6 +569,15 @@ impl Metadata23 {
             &self.get_version_escaped()
         ))
     }
+
+    /// Returns the name of the .data directory as defined in the wheel specification
+    pub fn get_data_dir(&self) -> PathBuf {
+        PathBuf::from(format!(
+            "{}-{}.data",
+            &self.get_distribution_escaped(),
+            &self.get_version_escaped()
+        ))
+    }
 }
 
 /// Escape email addresses with display name if necessary
@@ -611,7 +632,7 @@ mod test {
         readme: &str,
         cargo_toml: &str,
         expected: Expect,
-    ) -> Metadata23 {
+    ) -> Metadata24 {
         let crate_dir = tempfile::tempdir().unwrap();
         let crate_path = crate_dir.path();
         let manifest_path = crate_path.join("Cargo.toml");
@@ -635,7 +656,7 @@ mod test {
             .exec()
             .unwrap();
 
-        let metadata = Metadata23::from_cargo_toml(crate_path, &cargo_metadata).unwrap();
+        let metadata = Metadata24::from_cargo_toml(crate_path, &cargo_metadata).unwrap();
 
         let actual = metadata.to_file_contents().unwrap();
 
@@ -679,7 +700,7 @@ mod test {
         );
 
         let expected = expect![[r#"
-            Metadata-Version: 2.3
+            Metadata-Version: 2.4
             Name: info-project
             Version: 0.1.0
             Summary: A test project
@@ -727,7 +748,7 @@ mod test {
             .manifest_path(manifest_dir.join("Cargo.toml"))
             .exec()
             .unwrap();
-        let mut metadata = Metadata23::from_cargo_toml(&manifest_dir, &cargo_metadata).unwrap();
+        let mut metadata = Metadata24::from_cargo_toml(&manifest_dir, &cargo_metadata).unwrap();
         let pyproject_toml = PyProjectToml::new(manifest_dir.join("pyproject.toml")).unwrap();
         metadata
             .merge_pyproject_toml(&manifest_dir, &pyproject_toml)
@@ -776,7 +797,7 @@ mod test {
             .manifest_path(manifest_dir.join("Cargo.toml"))
             .exec()
             .unwrap();
-        let mut metadata = Metadata23::from_cargo_toml(&manifest_dir, &cargo_metadata).unwrap();
+        let mut metadata = Metadata24::from_cargo_toml(&manifest_dir, &cargo_metadata).unwrap();
         let pyproject_toml = PyProjectToml::new(manifest_dir.join("pyproject.toml")).unwrap();
         metadata
             .merge_pyproject_toml(&manifest_dir, &pyproject_toml)
@@ -797,7 +818,7 @@ mod test {
             .manifest_path(manifest_dir.join("Cargo.toml"))
             .exec()
             .unwrap();
-        let metadata = Metadata23::from_cargo_toml(&manifest_dir, &cargo_metadata).unwrap();
+        let metadata = Metadata24::from_cargo_toml(&manifest_dir, &cargo_metadata).unwrap();
         assert!(metadata.description.unwrap().starts_with("# pyo3-mixed"));
         assert_eq!(
             metadata.description_content_type.unwrap(),
@@ -812,7 +833,7 @@ mod test {
             .manifest_path(manifest_dir.join("Cargo.toml"))
             .exec()
             .unwrap();
-        let mut metadata = Metadata23::from_cargo_toml(&manifest_dir, &cargo_metadata).unwrap();
+        let mut metadata = Metadata24::from_cargo_toml(&manifest_dir, &cargo_metadata).unwrap();
         let pyproject_toml = PyProjectToml::new(manifest_dir.join("pyproject.toml")).unwrap();
         metadata
             .merge_pyproject_toml(&manifest_dir, &pyproject_toml)
