@@ -2,7 +2,7 @@
 use crate::project_layout::ProjectLayout;
 use crate::target::Os;
 use crate::{
-    pyproject_toml::Format, BridgeModel, Metadata23, PyProjectToml, PythonInterpreter, Target,
+    pyproject_toml::Format, BridgeModel, Metadata24, PyProjectToml, PythonInterpreter, Target,
 };
 use anyhow::{anyhow, bail, Context, Result};
 use base64::engine::general_purpose::URL_SAFE_NO_PAD;
@@ -18,6 +18,7 @@ use fs_err::OpenOptions;
 use ignore::overrides::Override;
 use ignore::WalkBuilder;
 use indexmap::IndexMap;
+use itertools::Itertools;
 use normpath::PathExt as _;
 use same_file::is_same_file;
 use sha2::{Digest, Sha256};
@@ -142,10 +143,10 @@ impl PathWriter {
     }
 
     /// Writes the RECORD file after everything else has been written
-    pub fn write_record(self, metadata23: &Metadata23) -> Result<()> {
+    pub fn write_record(self, metadata24: &Metadata24) -> Result<()> {
         let record_file = self
             .base_path
-            .join(metadata23.get_dist_info_dir())
+            .join(metadata24.get_dist_info_dir())
             .join("RECORD");
         let mut buffer = File::create(&record_file).context(format!(
             "Failed to create a file at {}",
@@ -185,7 +186,7 @@ impl ModuleWriter for PathWriter {
         target: impl AsRef<Path>,
         source: Option<&Path>,
         bytes: &[u8],
-        permissions: u32,
+        #[cfg_attr(target_os = "windows", allow(unused_variables))] permissions: u32,
     ) -> Result<()> {
         let path = self.base_path.join(&target);
 
@@ -269,7 +270,7 @@ impl ModuleWriter for WheelWriter {
             zip::CompressionMethod::Deflated
         };
 
-        let mut options = zip::write::FileOptions::default()
+        let mut options = zip::write::SimpleFileOptions::default()
             .unix_permissions(permissions)
             .compression_method(compression_method);
         let mtime = self.mtime().ok();
@@ -294,14 +295,14 @@ impl WheelWriter {
     pub fn new(
         tag: &str,
         wheel_dir: &Path,
-        metadata23: &Metadata23,
+        metadata24: &Metadata24,
         tags: &[String],
         excludes: Override,
     ) -> Result<WheelWriter> {
         let wheel_path = wheel_dir.join(format!(
             "{}-{}-{}.whl",
-            metadata23.get_distribution_escaped(),
-            metadata23.get_version_escaped(),
+            metadata24.get_distribution_escaped(),
+            metadata24.get_version_escaped(),
             tag
         ));
 
@@ -310,13 +311,13 @@ impl WheelWriter {
         let mut builder = WheelWriter {
             zip: ZipWriter::new(file),
             record: Vec::new(),
-            record_file: metadata23.get_dist_info_dir().join("RECORD"),
+            record_file: metadata24.get_dist_info_dir().join("RECORD"),
             wheel_path,
             file_tracker: FileTracker::default(),
             excludes,
         };
 
-        write_dist_info(&mut builder, metadata23, tags)?;
+        write_dist_info(&mut builder, metadata24, tags)?;
 
         Ok(builder)
     }
@@ -325,7 +326,7 @@ impl WheelWriter {
     pub fn add_pth(
         &mut self,
         project_layout: &ProjectLayout,
-        metadata23: &Metadata23,
+        metadata24: &Metadata24,
     ) -> Result<()> {
         if project_layout.python_module.is_some() || !project_layout.python_packages.is_empty() {
             let absolute_path = project_layout
@@ -339,7 +340,7 @@ impl WheelWriter {
                 })?
                 .into_path_buf();
             if let Some(python_path) = absolute_path.to_str() {
-                let name = metadata23.get_distribution_escaped();
+                let name = metadata24.get_distribution_escaped();
                 let target = format!("{name}.pth");
                 debug!("Adding {} from {}", target, python_path);
                 self.add_bytes(target, None, python_path.as_bytes())?;
@@ -378,7 +379,8 @@ impl WheelWriter {
             zip::CompressionMethod::Deflated
         };
 
-        let mut options = zip::write::FileOptions::default().compression_method(compression_method);
+        let mut options =
+            zip::write::SimpleFileOptions::default().compression_method(compression_method);
         let mtime = self.mtime().ok();
         if let Some(mtime) = mtime {
             options = options.last_modified_time(mtime);
@@ -471,7 +473,7 @@ impl SDistWriter {
     /// Create a source distribution .tar.gz which can be subsequently expanded
     pub fn new(
         wheel_dir: impl AsRef<Path>,
-        metadata23: &Metadata23,
+        metadata24: &Metadata24,
         excludes: Override,
     ) -> Result<Self, io::Error> {
         let path = wheel_dir
@@ -479,8 +481,8 @@ impl SDistWriter {
             .normalize()?
             .join(format!(
                 "{}-{}.tar.gz",
-                &metadata23.get_distribution_escaped(),
-                &metadata23.get_version_escaped()
+                &metadata24.get_distribution_escaped(),
+                &metadata24.get_version_escaped()
             ))
             .into_path_buf();
 
@@ -595,14 +597,14 @@ fn entry_points_txt(
 }
 
 /// Glue code that exposes `lib`.
-fn cffi_init_file(extension_name: &str) -> String {
+fn cffi_init_file(cffi_module_file_name: &str) -> String {
     format!(
         r#"__all__ = ["lib", "ffi"]
 
 import os
 from .ffi import ffi
 
-lib = ffi.dlopen(os.path.join(os.path.dirname(__file__), '{extension_name}.so'))
+lib = ffi.dlopen(os.path.join(os.path.dirname(__file__), '{cffi_module_file_name}'))
 del os
 "#
     )
@@ -791,6 +793,28 @@ fn handle_cffi_call_result(
     }
 }
 
+// Extract the shared object from a AIX big library archive
+fn unpack_big_archive(target: &Target, artifact: &Path, temp_dir_path: &Path) -> Result<PathBuf> {
+    // Newer rust generates archived dylibs on AIX, as shared
+    // libraries are typically archived on the platform.
+    if target.cross_compiling() {
+        bail!("can't unpack big archive format while cross_compiling")
+    }
+    debug!("Unpacking archive {}", artifact.display());
+    let mut ar_command = Command::new("ar");
+    ar_command
+        .current_dir(temp_dir_path)
+        .arg("-X64")
+        .arg("x")
+        .arg(artifact);
+    let status = ar_command.status().expect("Failed to run ar");
+    if !status.success() {
+        bail!(r#"ar finished with "{}": `{:?}`"#, status, ar_command,)
+    }
+    let unpacked_artifact = temp_dir_path.join(artifact.with_extension("so").file_name().unwrap());
+    Ok(unpacked_artifact)
+}
+
 /// Copies the shared library into the module, which is the only extra file needed with bindings
 #[allow(clippy::too_many_arguments)]
 #[instrument(skip_all)]
@@ -823,6 +847,28 @@ pub fn write_bindings_module(
         python_interpreter.get_library_name(ext_name)
     };
 
+    let artifact_is_big_ar =
+        target.is_aix() && artifact.extension().unwrap_or(OsStr::new(" ")) == OsStr::new("a");
+    let temp_dir = if artifact_is_big_ar {
+        Some(tempfile::tempdir()?)
+    } else {
+        None
+    };
+    let artifact_buff = if artifact_is_big_ar {
+        Some(unpack_big_archive(
+            target,
+            artifact,
+            temp_dir.as_ref().unwrap().path(),
+        )?)
+    } else {
+        None
+    };
+    let artifact = if artifact_is_big_ar {
+        artifact_buff.as_ref().unwrap()
+    } else {
+        artifact
+    };
+
     if !editable {
         write_python_part(writer, project_layout, pyproject_toml)
             .context("Failed to add the python module to the package")?;
@@ -880,6 +926,7 @@ if hasattr({ext_name}, "__all__"):
 #[allow(clippy::too_many_arguments)]
 pub fn write_cffi_module(
     writer: &mut impl ModuleWriter,
+    target: &Target,
     project_layout: &ProjectLayout,
     crate_dir: &Path,
     target_dir: &Path,
@@ -896,22 +943,28 @@ pub fn write_cffi_module(
             .context("Failed to add the python module to the package")?;
     }
 
+    let cffi_module_file_name = {
+        let extension_name = &project_layout.extension_name;
+        // https://cffi.readthedocs.io/en/stable/embedding.html#issues-about-using-the-so
+        match target.target_os() {
+            Os::Macos => format!("lib{extension_name}.dylib"),
+            Os::Windows => format!("{extension_name}.dll"),
+            _ => format!("lib{extension_name}.so"),
+        }
+    };
     let module;
     if let Some(python_module) = &project_layout.python_module {
         if editable {
             let base_path = python_module.join(&project_layout.extension_name);
             fs::create_dir_all(&base_path)?;
-            let target = base_path.join(format!(
-                "{extension_name}.so",
-                extension_name = &project_layout.extension_name
-            ));
+            let target = base_path.join(&cffi_module_file_name);
             fs::copy(artifact, &target).context(format!(
                 "Failed to copy {} to {}",
                 artifact.display(),
                 target.display()
             ))?;
             File::create(base_path.join("__init__.py"))?
-                .write_all(cffi_init_file(&project_layout.extension_name).as_bytes())?;
+                .write_all(cffi_init_file(&cffi_module_file_name).as_bytes())?;
             File::create(base_path.join("ffi.py"))?.write_all(cffi_declarations.as_bytes())?;
         }
 
@@ -940,17 +993,10 @@ pub fn write_cffi_module(
         writer.add_bytes(
             module.join("__init__.py"),
             None,
-            cffi_init_file(&project_layout.extension_name).as_bytes(),
+            cffi_init_file(&cffi_module_file_name).as_bytes(),
         )?;
         writer.add_bytes(module.join("ffi.py"), None, cffi_declarations.as_bytes())?;
-        writer.add_file_with_permissions(
-            module.join(format!(
-                "{extension_name}.so",
-                extension_name = &project_layout.extension_name
-            )),
-            artifact,
-            0o755,
-        )?;
+        writer.add_file_with_permissions(module.join(&cffi_module_file_name), artifact, 0o755)?;
     }
 
     Ok(())
@@ -971,7 +1017,7 @@ struct UniFfiBindingsConfig {
 
 #[derive(Debug, Clone)]
 struct UniFfiBindings {
-    name: String,
+    names: Vec<String>,
     cdylib: String,
     path: PathBuf,
 }
@@ -1007,14 +1053,18 @@ fn uniffi_bindgen_command(crate_dir: &Path) -> Result<Command> {
 
     let command = if has_uniffi_bindgen_target {
         let mut command = Command::new("cargo");
-        command.args(["run", "--bin", "uniffi-bindgen", "--manifest-path"]);
-        command.arg(manifest_path);
-        command.current_dir(crate_dir);
+        command
+            .args(["run", "--bin", "uniffi-bindgen", "--manifest-path"])
+            .arg(manifest_path)
+            .current_dir(crate_dir)
+            .env_remove("CARGO_BUILD_TARGET");
         command
     } else if has_uniffi_bindgen_workspace_package {
         let mut command = Command::new("cargo");
-        command.args(["run", "--bin", "uniffi-bindgen"]);
-        command.current_dir(cargo_metadata.workspace_root);
+        command
+            .args(["run", "--bin", "uniffi-bindgen"])
+            .current_dir(cargo_metadata.workspace_root)
+            .env_remove("CARGO_BUILD_TARGET");
         command
     } else {
         let mut command = Command::new("uniffi-bindgen");
@@ -1027,6 +1077,7 @@ fn uniffi_bindgen_command(crate_dir: &Path) -> Result<Command> {
 fn generate_uniffi_bindings(
     crate_dir: &Path,
     target_dir: &Path,
+    module_name: &str,
     target_os: Os,
     artifact: &Path,
 ) -> Result<UniFfiBindings> {
@@ -1036,6 +1087,7 @@ fn generate_uniffi_bindings(
         .normalize()?
         .join("maturin")
         .join("uniffi")
+        .join(module_name)
         .into_path_buf();
     fs::create_dir_all(&binding_dir)?;
 
@@ -1111,8 +1163,20 @@ fn generate_uniffi_bindings(
         _ => format!("lib{cdylib_name}.so"),
     };
 
+    let py_bindings = fs::read_dir(&binding_dir)?
+        .flatten()
+        .filter(|file| file.path().extension().and_then(std::ffi::OsStr::to_str) == Some("py"))
+        .map(|file| {
+            file.path()
+                .file_stem()
+                .unwrap()
+                .to_string_lossy()
+                .to_string()
+        })
+        .collect_vec();
+
     Ok(UniFfiBindings {
-        name: py_binding_name,
+        names: py_bindings,
         cdylib,
         path: binding_dir,
     })
@@ -1132,11 +1196,16 @@ pub fn write_uniffi_module(
     pyproject_toml: Option<&PyProjectToml>,
 ) -> Result<()> {
     let UniFfiBindings {
-        name: binding_name,
+        names: binding_names,
         cdylib,
         path: binding_dir,
-    } = generate_uniffi_bindings(crate_dir, target_dir, target_os, artifact)?;
-    let py_init = format!("from .{binding_name} import *  # NOQA\n");
+    } = generate_uniffi_bindings(crate_dir, target_dir, module_name, target_os, artifact)?;
+
+    let py_init = binding_names
+        .iter()
+        .map(|name| format!("from .{name} import *  # NOQA\n"))
+        .collect::<Vec<String>>()
+        .join("");
 
     if !editable {
         write_python_part(writer, project_layout, pyproject_toml)
@@ -1156,13 +1225,13 @@ pub fn write_uniffi_module(
             ))?;
 
             File::create(base_path.join("__init__.py"))?.write_all(py_init.as_bytes())?;
-            if let Ok(read_dir) = fs::read_dir(&binding_dir) {
-                for binding_file in read_dir.flatten() {
-                    let target: PathBuf = base_path.join(binding_file.file_name());
-                    fs::copy(binding_file.path(), &target).with_context(|| {
-                        format!("Failed to copy {:?} to {:?}", binding_file.path(), target)
+
+            for binding_name in binding_names.iter() {
+                let target: PathBuf = base_path.join(binding_name).with_extension("py");
+                fs::copy(binding_dir.join(binding_name).with_extension("py"), &target)
+                    .with_context(|| {
+                        format!("Failed to copy {:?} to {:?}", binding_dir.display(), target)
                     })?;
-                }
             }
         }
 
@@ -1189,10 +1258,11 @@ pub fn write_uniffi_module(
 
     if !editable || project_layout.python_module.is_none() {
         writer.add_bytes(module.join("__init__.py"), None, py_init.as_bytes())?;
-        if let Ok(read_dir) = fs::read_dir(binding_dir) {
-            for binding_file in read_dir.flatten() {
-                writer.add_file(module.join(binding_file.file_name()), binding_file.path())?;
-            }
+        for binding in binding_names.iter() {
+            writer.add_file(
+                module.join(binding).with_extension("py"),
+                binding_dir.join(binding).with_extension("py"),
+            )?;
         }
         writer.add_file_with_permissions(module.join(cdylib), artifact, 0o755)?;
     }
@@ -1204,7 +1274,7 @@ pub fn write_uniffi_module(
 pub fn write_bin(
     writer: &mut impl ModuleWriter,
     artifact: &Path,
-    metadata: &Metadata23,
+    metadata: &Metadata24,
     bin_name: &str,
 ) -> Result<()> {
     let data_dir = PathBuf::from(format!(
@@ -1226,7 +1296,7 @@ pub fn write_bin(
 /// Note that the wasm binary needs to be written separately by [write_bin]
 pub fn write_wasm_launcher(
     writer: &mut impl ModuleWriter,
-    metadata: &Metadata23,
+    metadata: &Metadata24,
     bin_name: &str,
 ) -> Result<()> {
     let entrypoint_script = format!(
@@ -1363,17 +1433,17 @@ pub fn write_python_part(
 /// Creates the .dist-info directory and fills it with all metadata files except RECORD
 pub fn write_dist_info(
     writer: &mut impl ModuleWriter,
-    metadata23: &Metadata23,
+    metadata24: &Metadata24,
     tags: &[String],
 ) -> Result<()> {
-    let dist_info_dir = metadata23.get_dist_info_dir();
+    let dist_info_dir = metadata24.get_dist_info_dir();
 
     writer.add_directory(&dist_info_dir)?;
 
     writer.add_bytes(
         dist_info_dir.join("METADATA"),
         None,
-        metadata23.to_file_contents()?.as_bytes(),
+        metadata24.to_file_contents()?.as_bytes(),
     )?;
 
     writer.add_bytes(
@@ -1383,13 +1453,13 @@ pub fn write_dist_info(
     )?;
 
     let mut entry_points = String::new();
-    if !metadata23.scripts.is_empty() {
-        entry_points.push_str(&entry_points_txt("console_scripts", &metadata23.scripts));
+    if !metadata24.scripts.is_empty() {
+        entry_points.push_str(&entry_points_txt("console_scripts", &metadata24.scripts));
     }
-    if !metadata23.gui_scripts.is_empty() {
-        entry_points.push_str(&entry_points_txt("gui_scripts", &metadata23.gui_scripts));
+    if !metadata24.gui_scripts.is_empty() {
+        entry_points.push_str(&entry_points_txt("gui_scripts", &metadata24.gui_scripts));
     }
-    for (entry_type, scripts) in &metadata23.entry_points {
+    for (entry_type, scripts) in &metadata24.entry_points {
         entry_points.push_str(&entry_points_txt(entry_type, scripts));
     }
     if !entry_points.is_empty() {
@@ -1400,10 +1470,10 @@ pub fn write_dist_info(
         )?;
     }
 
-    if !metadata23.license_files.is_empty() {
+    if !metadata24.license_files.is_empty() {
         let license_files_dir = dist_info_dir.join("licenses");
         writer.add_directory(&license_files_dir)?;
-        for path in &metadata23.license_files {
+        for path in &metadata24.license_files {
             let filename = path.file_name().with_context(|| {
                 format!("missing file name for license file {}", path.display())
             })?;
@@ -1419,7 +1489,11 @@ pub fn write_dist_info(
 /// to save or generate the data in other places
 ///
 /// See https://peps.python.org/pep-0427/#file-contents
-pub fn add_data(writer: &mut impl ModuleWriter, data: Option<&Path>) -> Result<()> {
+pub fn add_data(
+    writer: &mut impl ModuleWriter,
+    metadata24: &Metadata24,
+    data: Option<&Path>,
+) -> Result<()> {
     let possible_data_dir_names = ["data", "scripts", "headers", "purelib", "platlib"];
     if let Some(data) = data {
         for subdir in fs::read_dir(data).context("Failed to read data dir")? {
@@ -1447,7 +1521,9 @@ pub fn add_data(writer: &mut impl ModuleWriter, data: Option<&Path>) -> Result<(
                     let mode = file.metadata()?.permissions().mode();
                     #[cfg(not(unix))]
                     let mode = 0o644;
-                    let relative = file.path().strip_prefix(data.parent().unwrap()).unwrap();
+                    let relative = metadata24
+                        .get_data_dir()
+                        .join(file.path().strip_prefix(data).unwrap());
 
                     if file.path_is_symlink() {
                         // Copy the actual file contents, not the link, so that you can create a
@@ -1484,7 +1560,7 @@ mod tests {
     #[test]
     // The mechanism is the same for wheel_writer
     fn sdist_writer_excludes() -> Result<(), Box<dyn std::error::Error>> {
-        let metadata = Metadata23::new("dummy".to_string(), Version::new([1, 0]));
+        let metadata = Metadata24::new("dummy".to_string(), Version::new([1, 0]));
         let perm = 0o777;
 
         // No excludes
