@@ -97,13 +97,50 @@ impl GlobPattern {
 pub struct CargoTarget {
     /// Name as given in the `Cargo.toml` or generated from the file name
     pub name: String,
-    /// Kind of target ("bin", "lib")
-    pub kind: Option<String>,
+    /// Kind of target ("bin", "cdylib")
+    pub kind: Option<CargoCrateType>,
     // TODO: Add bindings option
     // Bridge model, which kind of bindings to use
     // pub bindings: Option<String>,
 }
 
+/// Supported cargo crate types
+#[derive(Serialize, Deserialize, Debug, Clone, Copy)]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
+pub enum CargoCrateType {
+    /// Binary executable target
+    #[serde(rename = "bin")]
+    Bin,
+    /// Dynamic system library target
+    #[serde(rename = "cdylib")]
+    CDyLib,
+    /// Dynamic Rust library target
+    #[serde(rename = "dylib")]
+    DyLib,
+    /// Rust library
+    #[serde(rename = "lib")]
+    Lib,
+    /// Rust library for use as an intermediate target
+    #[serde(rename = "rlib")]
+    RLib,
+    /// Static library
+    #[serde(rename = "staticlib")]
+    StaticLib,
+}
+
+impl From<CargoCrateType> for cargo_metadata::CrateType {
+    fn from(value: CargoCrateType) -> Self {
+        match value {
+            CargoCrateType::Bin => cargo_metadata::CrateType::Bin,
+            CargoCrateType::CDyLib => cargo_metadata::CrateType::CDyLib,
+            CargoCrateType::DyLib => cargo_metadata::CrateType::DyLib,
+            CargoCrateType::Lib => cargo_metadata::CrateType::Lib,
+            CargoCrateType::RLib => cargo_metadata::CrateType::RLib,
+            CargoCrateType::StaticLib => cargo_metadata::CrateType::StaticLib,
+        }
+    }
+}
+
 /// Target configuration
 #[derive(Serialize, Deserialize, Debug, Clone)]
 #[serde(rename_all = "kebab-case")]
@@ -389,6 +426,30 @@ impl PyProjectToml {
         );
         false
     }
+
+    /// Having a pyproject.toml project table with neither `version` nor `dynamic = ['version']`
+    /// violates https://packaging.python.org/en/latest/specifications/pyproject-toml/#dynamic.
+    ///
+    /// Returns true if version information is specified correctly or no project table is present.
+    pub fn warn_invalid_version_info(&self) -> bool {
+        let Some(project) = &self.project else {
+            return true;
+        };
+        let has_static_version = project.version.is_some();
+        let has_dynamic_version = project
+            .dynamic
+            .as_ref()
+            .is_some_and(|d| d.iter().any(|s| s == "version"));
+        if has_static_version && has_dynamic_version {
+            eprintln!("⚠️  Warning: `project.dynamic` must not specify `version` when `project.version` is present in pyproject.toml");
+            return false;
+        }
+        if !has_static_version && !has_dynamic_version {
+            eprintln!("⚠️  Warning: `project.version` field is required in pyproject.toml unless it is present in the `project.dynamic` list");
+            return false;
+        }
+        true
+    }
 }
 
 #[cfg(test)]
@@ -507,6 +568,62 @@ mod tests {
         assert!(!without_constraint.warn_bad_maturin_version());
     }
 
+    #[test]
+    fn test_warn_invalid_version_info_conflict() {
+        let conflict = toml::from_str::<PyProjectToml>(
+            r#"[build-system]
+            requires = ["maturin==1.0.0"]
+
+            [project]
+            name = "..."
+            version = "1.2.3"
+            dynamic = ['version']
+            "#,
+        )
+        .unwrap();
+        assert!(!conflict.warn_invalid_version_info());
+    }
+
+    #[test]
+    fn test_warn_invalid_version_info_missing() {
+        let missing = toml::from_str::<PyProjectToml>(
+            r#"[build-system]
+            requires = ["maturin==1.0.0"]
+
+            [project]
+            name = "..."
+            "#,
+        )
+        .unwrap();
+        assert!(!missing.warn_invalid_version_info());
+    }
+
+    #[test]
+    fn test_warn_invalid_version_info_ok() {
+        let static_ver = toml::from_str::<PyProjectToml>(
+            r#"[build-system]
+            requires = ["maturin==1.0.0"]
+
+            [project]
+            name = "..."
+            version = "1.2.3"
+            "#,
+        )
+        .unwrap();
+        assert!(static_ver.warn_invalid_version_info());
+        let dynamic_ver = toml::from_str::<PyProjectToml>(
+            r#"[build-system]
+            requires = ["maturin==1.0.0"]
+
+            [project]
+            name = "..."
+            dynamic = ['version']
+            "#,
+        )
+        .unwrap();
+        assert!(dynamic_ver.warn_invalid_version_info());
+    }
+
     #[test]
     fn deserialize_include_exclude() {
         let single = r#"include = ["single"]"#;
@@ -590,11 +707,13 @@ mod tests {
         let inner_error = outer_error.source().unwrap();
 
         let expected = expect![[r#"
-            TOML parse error at line 7, column 17
-              |
-            7 | license-files = [ "license.txt",]
-              |                 ^^^^^^^^^^^^^^^^^
-            wanted string or table
+            TOML parse error at line 10, column 16
+               |
+            10 | dependencies = [ "packaging", "...",]
+               |                ^^^^^^^^^^^^^^^^^^^^^^
+            URL requirement must be preceded by a package name. Add the name of the package before the URL (e.g., `package_name @ /path/to/file`).
+            ...
+            ^^^
         "#]];
         expected.assert_eq(&inner_error.to_string());
     }
