@@ -1,4 +1,7 @@
-use super::{InterpreterKind, MAXIMUM_PYPY_MINOR, MAXIMUM_PYTHON_MINOR, MINIMUM_PYTHON_MINOR};
+use super::{
+    InterpreterKind, MAXIMUM_PYPY_MINOR, MAXIMUM_PYTHON_MINOR, MINIMUM_PYPY_MINOR,
+    MINIMUM_PYTHON_MINOR,
+};
 use crate::target::{Arch, Os};
 use crate::Target;
 use anyhow::{format_err, Context, Result};
@@ -22,8 +25,11 @@ pub struct InterpreterConfig {
     #[serde(rename = "interpreter")]
     pub interpreter_kind: InterpreterKind,
     /// For linux and mac, this contains the value of the abiflags, e.g. "m"
-    /// for python3.7m or "dm" for python3.6dm. Since python3.8, the value is
-    /// empty. On windows, the value was always None.
+    /// for python3.7m or "dm" for python3.6dm.
+    ///
+    /// * Since python3.8, the value is empty
+    /// * Since python3.13, the value is "t" for free-threaded builds.
+    /// * On Windows, the value was always None.
     ///
     /// See PEP 261 and PEP 393 for details
     pub abiflags: String,
@@ -31,6 +37,8 @@ pub struct InterpreterConfig {
     pub ext_suffix: String,
     /// Pointer width
     pub pointer_width: Option<usize>,
+    /// Is GIL disabled, i.e. free-threaded build
+    pub gil_disabled: bool,
 }
 
 impl InterpreterConfig {
@@ -39,6 +47,7 @@ impl InterpreterConfig {
         target: &Target,
         python_impl: InterpreterKind,
         python_version: (usize, usize),
+        abiflags: &str,
     ) -> Option<Self> {
         use InterpreterKind::*;
 
@@ -49,18 +58,17 @@ impl InterpreterConfig {
         }
         let python_ext_arch = target.get_python_ext_arch(python_impl);
         let target_env = target.get_python_target_env(python_impl, python_version);
+        let gil_disabled = abiflags == "t";
         match (target.target_os(), python_impl) {
             (Os::Linux, CPython) => {
                 let abiflags = if python_version < (3, 8) {
                     "m".to_string()
                 } else {
-                    "".to_string()
+                    abiflags.to_string()
                 };
-                let ldversion = format!("{}{}{}", major, minor, abiflags);
-                let ext_suffix = format!(
-                    ".cpython-{}-{}-linux-{}.so",
-                    ldversion, python_ext_arch, target_env
-                );
+                let ldversion = format!("{major}{minor}{abiflags}");
+                let ext_suffix =
+                    format!(".cpython-{ldversion}-{python_ext_arch}-linux-{target_env}.so");
                 Some(Self {
                     major,
                     minor,
@@ -68,12 +76,12 @@ impl InterpreterConfig {
                     abiflags,
                     ext_suffix,
                     pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
                 })
             }
             (Os::Linux, PyPy) => {
-                let abi_tag = format!("pypy{}{}-{}", major, minor, PYPY_ABI_TAG);
-                let ext_suffix =
-                    format!(".{}-{}-linux-{}.so", abi_tag, python_ext_arch, target_env);
+                let abi_tag = format!("pypy{major}{minor}-{PYPY_ABI_TAG}");
+                let ext_suffix = format!(".{abi_tag}-{python_ext_arch}-linux-{target_env}.so");
                 Some(Self {
                     major,
                     minor,
@@ -81,16 +89,17 @@ impl InterpreterConfig {
                     abiflags: String::new(),
                     ext_suffix,
                     pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
                 })
             }
             (Os::Macos, CPython) => {
                 let abiflags = if python_version < (3, 8) {
                     "m".to_string()
                 } else {
-                    "".to_string()
+                    abiflags.to_string()
                 };
-                let ldversion = format!("{}{}{}", major, minor, abiflags);
-                let ext_suffix = format!(".cpython-{}-darwin.so", ldversion);
+                let ldversion = format!("{major}{minor}{abiflags}");
+                let ext_suffix = format!(".cpython-{ldversion}-darwin.so");
                 Some(Self {
                     major,
                     minor,
@@ -98,10 +107,11 @@ impl InterpreterConfig {
                     abiflags,
                     ext_suffix,
                     pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
                 })
             }
             (Os::Macos, PyPy) => {
-                let ext_suffix = format!(".pypy{}{}-{}-darwin.so", major, minor, PYPY_ABI_TAG);
+                let ext_suffix = format!(".pypy{major}{minor}-{PYPY_ABI_TAG}-darwin.so");
                 Some(Self {
                     major,
                     minor,
@@ -109,9 +119,15 @@ impl InterpreterConfig {
                     abiflags: String::new(),
                     ext_suffix,
                     pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
                 })
             }
             (Os::Windows, CPython) => {
+                let abiflags = if python_version < (3, 8) {
+                    "m".to_string()
+                } else {
+                    abiflags.to_string()
+                };
                 let ext_suffix = if python_version < (3, 8) {
                     ".pyd".to_string()
                 } else {
@@ -121,37 +137,7 @@ impl InterpreterConfig {
                         Arch::X86_64 => "win_amd64",
                         _ => return None,
                     };
-                    format!(".cp{}{}-{}.pyd", major, minor, platform)
-                };
-                Some(Self {
-                    major,
-                    minor,
-                    interpreter_kind: CPython,
-                    abiflags: String::new(),
-                    ext_suffix,
-                    pointer_width: Some(target.pointer_width()),
-                })
-            }
-            (Os::Windows, PyPy) => {
-                if target.target_arch() != Arch::X86_64 {
-                    // PyPy on Windows only supports x86_64
-                    return None;
-                }
-                let ext_suffix = format!(".pypy{}{}-{}-win_amd64.pyd", major, minor, PYPY_ABI_TAG);
-                Some(Self {
-                    major,
-                    minor,
-                    interpreter_kind: PyPy,
-                    abiflags: String::new(),
-                    ext_suffix,
-                    pointer_width: Some(target.pointer_width()),
-                })
-            }
-            (Os::FreeBsd, CPython) => {
-                let (abiflags, ext_suffix) = if python_version < (3, 8) {
-                    ("m".to_string(), ".so".to_string())
-                } else {
-                    ("".to_string(), format!(".cpython-{}{}.so", major, minor))
+                    format!(".cp{major}{minor}{abiflags}-{platform}.pyd")
                 };
                 Some(Self {
                     major,
@@ -160,6 +146,42 @@ impl InterpreterConfig {
                     abiflags,
                     ext_suffix,
                     pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
+                })
+            }
+            (Os::Windows, PyPy) => {
+                if target.target_arch() != Arch::X86_64 {
+                    // PyPy on Windows only supports x86_64
+                    return None;
+                }
+                let ext_suffix = format!(".pypy{major}{minor}-{PYPY_ABI_TAG}-win_amd64.pyd");
+                Some(Self {
+                    major,
+                    minor,
+                    interpreter_kind: PyPy,
+                    abiflags: String::new(),
+                    ext_suffix,
+                    pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
+                })
+            }
+            (Os::FreeBsd, CPython) => {
+                let (abiflags, ext_suffix) = if python_version < (3, 8) {
+                    ("m".to_string(), ".so".to_string())
+                } else {
+                    (
+                        abiflags.to_string(),
+                        format!(".cpython-{major}{minor}{abiflags}.so"),
+                    )
+                };
+                Some(Self {
+                    major,
+                    minor,
+                    interpreter_kind: CPython,
+                    abiflags,
+                    ext_suffix,
+                    pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
                 })
             }
             (Os::NetBsd, CPython) => {
@@ -171,11 +193,12 @@ impl InterpreterConfig {
                     abiflags: String::new(),
                     ext_suffix,
                     pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
                 })
             }
             (Os::OpenBsd, CPython) => {
-                let ldversion = format!("{}{}", major, minor);
-                let ext_suffix = format!(".cpython-{}.so", ldversion);
+                let ldversion = format!("{major}{minor}");
+                let ext_suffix = format!(".cpython-{ldversion}{abiflags}.so");
                 Some(Self {
                     major,
                     minor,
@@ -183,12 +206,12 @@ impl InterpreterConfig {
                     abiflags: String::new(),
                     ext_suffix,
                     pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
                 })
             }
             (Os::Emscripten, CPython) => {
-                let ldversion = format!("{}{}", major, minor);
-                let ext_suffix =
-                    format!(".cpython-{}-{}-emscripten.so", ldversion, python_ext_arch);
+                let ldversion = format!("{major}{minor}");
+                let ext_suffix = format!(".cpython-{ldversion}-{python_ext_arch}-emscripten.so");
                 Some(Self {
                     major,
                     minor,
@@ -196,6 +219,7 @@ impl InterpreterConfig {
                     abiflags: String::new(),
                     ext_suffix,
                     pointer_width: Some(target.pointer_width()),
+                    gil_disabled,
                 })
             }
             (_, _) => None,
@@ -205,12 +229,25 @@ impl InterpreterConfig {
     /// Lookup wellknown sysconfigs for a given target
     pub fn lookup_target(target: &Target) -> Vec<Self> {
         let mut configs = Vec::new();
-        for (python_impl, max_minor_ver) in [
-            (InterpreterKind::CPython, MAXIMUM_PYTHON_MINOR),
-            (InterpreterKind::PyPy, MAXIMUM_PYPY_MINOR),
+        for (python_impl, min_minor_ver, max_minor_ver) in [
+            (
+                InterpreterKind::CPython,
+                MINIMUM_PYTHON_MINOR,
+                MAXIMUM_PYTHON_MINOR,
+            ),
+            (
+                InterpreterKind::PyPy,
+                MINIMUM_PYPY_MINOR,
+                MAXIMUM_PYPY_MINOR,
+            ),
         ] {
-            for minor in MINIMUM_PYTHON_MINOR..=max_minor_ver {
-                if let Some(config) = Self::lookup_one(target, python_impl, (3, minor)) {
+            for minor in min_minor_ver..=max_minor_ver {
+                if let Some(config) = Self::lookup_one(target, python_impl, (3, minor), "") {
+                    configs.push(config);
+                }
+            }
+            for minor in 13..=max_minor_ver {
+                if let Some(config) = Self::lookup_one(target, python_impl, (3, minor), "t") {
                     configs.push(config);
                 }
             }
@@ -243,6 +280,7 @@ impl InterpreterConfig {
         let mut ext_suffix = None;
         let mut abi_tag = None;
         let mut pointer_width = None;
+        let mut build_flags: Option<String> = None;
 
         for (i, line) in lines.enumerate() {
             let line = line.context("failed to read line from config")?;
@@ -256,6 +294,7 @@ impl InterpreterConfig {
                 "ext_suffix" => parse_value!(ext_suffix, value),
                 "abi_tag" => parse_value!(abi_tag, value),
                 "pointer_width" => parse_value!(pointer_width, value),
+                "build_flags" => parse_value!(build_flags, value),
                 _ => continue,
             }
         }
@@ -283,7 +322,7 @@ impl InterpreterConfig {
             InterpreterKind::GraalPy => abi_tag.unwrap_or_else(|| GRAALPY_ABI_TAG.to_string()),
         };
         let file_ext = if target.is_windows() { "pyd" } else { "so" };
-        let ext_suffix = if target.is_linux() || target.is_macos() {
+        let ext_suffix = if target.is_linux() || target.is_macos() || target.is_hurd() {
             let target_env = target.get_python_target_env(interpreter_kind, (major, minor));
             match interpreter_kind {
                 InterpreterKind::CPython => ext_suffix.unwrap_or_else(|| {
@@ -334,6 +373,9 @@ impl InterpreterConfig {
         } else {
             ext_suffix.context("missing value for ext_suffix")?
         };
+        let gil_disabled = build_flags
+            .map(|flags| flags.contains("Py_GIL_DISABLED"))
+            .unwrap_or(false);
         Ok(Self {
             major,
             minor,
@@ -341,17 +383,23 @@ impl InterpreterConfig {
             abiflags: abiflags.unwrap_or_default(),
             ext_suffix,
             pointer_width,
+            gil_disabled,
         })
     }
 
     /// Generate pyo3 config file content
     pub fn pyo3_config_file(&self) -> String {
+        let build_flags = if self.gil_disabled {
+            "Py_GIL_DISABLED"
+        } else {
+            ""
+        };
         let mut content = format!(
             r#"implementation={implementation}
 version={major}.{minor}
 shared=true
 abi3=false
-build_flags=WITH_THREAD
+build_flags={build_flags}
 suppress_build_script_link_lines=false"#,
             implementation = self.interpreter_kind,
             major = self.major,
@@ -377,6 +425,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-x86_64-linux-gnu.so");
@@ -385,6 +434,7 @@ mod test {
             &Target::from_target_triple(Some("i686-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-i386-linux-gnu.so");
@@ -393,6 +443,7 @@ mod test {
             &Target::from_target_triple(Some("aarch64-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-aarch64-linux-gnu.so");
@@ -401,6 +452,7 @@ mod test {
             &Target::from_target_triple(Some("armv7-unknown-linux-gnueabihf".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-arm-linux-gnueabihf.so");
@@ -409,6 +461,7 @@ mod test {
             &Target::from_target_triple(Some("arm-unknown-linux-gnueabihf".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-arm-linux-gnueabihf.so");
@@ -417,6 +470,7 @@ mod test {
             &Target::from_target_triple(Some("powerpc64le-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(
@@ -428,6 +482,7 @@ mod test {
             &Target::from_target_triple(Some("powerpc-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-powerpc-linux-gnu.so");
@@ -436,6 +491,7 @@ mod test {
             &Target::from_target_triple(Some("mips64-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(
@@ -447,6 +503,7 @@ mod test {
             &Target::from_target_triple(Some("mips-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-mips-linux-gnu.so");
@@ -455,6 +512,7 @@ mod test {
             &Target::from_target_triple(Some("s390x-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-s390x-linux-gnu.so");
@@ -464,6 +522,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.abiflags, "");
@@ -473,6 +532,7 @@ mod test {
             &Target::from_target_triple(Some("i686-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".pypy39-pp73-x86-linux-gnu.so");
@@ -481,6 +541,7 @@ mod test {
             &Target::from_target_triple(Some("aarch64-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".pypy39-pp73-aarch64-linux-gnu.so");
@@ -489,6 +550,7 @@ mod test {
             &Target::from_target_triple(Some("armv7-unknown-linux-gnueabihf".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".pypy39-pp73-arm-linux-gnu.so");
@@ -497,6 +559,7 @@ mod test {
             &Target::from_target_triple(Some("arm-unknown-linux-gnueabihf".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".pypy39-pp73-arm-linux-gnu.so");
@@ -505,6 +568,7 @@ mod test {
             &Target::from_target_triple(Some("powerpc64le-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".pypy39-pp73-ppc_64-linux-gnu.so");
@@ -513,6 +577,7 @@ mod test {
             &Target::from_target_triple(Some("s390x-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".pypy39-pp73-s390x-linux-gnu.so");
@@ -525,6 +590,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-apple-darwin".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-darwin.so");
@@ -533,6 +599,7 @@ mod test {
             &Target::from_target_triple(Some("aarch64-apple-darwin".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310-darwin.so");
@@ -541,6 +608,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-apple-darwin".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 7),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.abiflags, "m");
@@ -551,6 +619,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-apple-darwin".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.abiflags, "");
@@ -560,6 +629,7 @@ mod test {
             &Target::from_target_triple(Some("aarch64-apple-darwin".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".pypy39-pp73-darwin.so");
@@ -572,6 +642,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-pc-windows-msvc".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cp310-win_amd64.pyd");
@@ -580,6 +651,7 @@ mod test {
             &Target::from_target_triple(Some("i686-pc-windows-msvc".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cp310-win32.pyd");
@@ -588,6 +660,7 @@ mod test {
             &Target::from_target_triple(Some("aarch64-pc-windows-msvc".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cp310-win_arm64.pyd");
@@ -597,6 +670,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-pc-windows-msvc".to_string())).unwrap(),
             InterpreterKind::PyPy,
             (3, 9),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".pypy39-pp73-win_amd64.pyd");
@@ -609,6 +683,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-unknown-freebsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 7),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.abiflags, "m");
@@ -618,6 +693,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-unknown-freebsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.abiflags, "");
@@ -627,6 +703,7 @@ mod test {
             &Target::from_target_triple(Some("i686-unknown-freebsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310.so");
@@ -635,6 +712,7 @@ mod test {
             &Target::from_target_triple(Some("aarch64-unknown-freebsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310.so");
@@ -643,6 +721,7 @@ mod test {
             &Target::from_target_triple(Some("armv7-unknown-freebsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310.so");
@@ -655,6 +734,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-unknown-netbsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 7),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.abiflags, "");
@@ -664,6 +744,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-unknown-netbsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".so");
@@ -676,6 +757,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-unknown-openbsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310.so");
@@ -684,6 +766,7 @@ mod test {
             &Target::from_target_triple(Some("i686-unknown-openbsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310.so");
@@ -692,6 +775,7 @@ mod test {
             &Target::from_target_triple(Some("aarch64-unknown-openbsd".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-310.so");
@@ -704,6 +788,7 @@ mod test {
             &Target::from_target_triple(Some("wasm32-unknown-emscripten".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.abiflags, "");
@@ -716,6 +801,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-unknown-linux-gnu".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 10),
+            "",
         )
         .unwrap();
         let config_file = sysconfig.pyo3_config_file();
@@ -724,7 +810,29 @@ mod test {
             version=3.10
             shared=true
             abi3=false
-            build_flags=WITH_THREAD
+            build_flags=
+            suppress_build_script_link_lines=false
+            pointer_width=64"#]];
+        expected.assert_eq(&config_file);
+    }
+
+    #[test]
+    fn test_pyo3_config_file_free_threaded_python_3_13() {
+        let sysconfig = InterpreterConfig::lookup_one(
+            &Target::from_target_triple(Some("x86_64-unknown-linux-gnu".to_string())).unwrap(),
+            InterpreterKind::CPython,
+            (3, 13),
+            "t",
+        )
+        .unwrap();
+        assert_eq!(sysconfig.ext_suffix, ".cpython-313t-x86_64-linux-gnu.so");
+        let config_file = sysconfig.pyo3_config_file();
+        let expected = expect![[r#"
+            implementation=CPython
+            version=3.13
+            shared=true
+            abi3=false
+            build_flags=Py_GIL_DISABLED
             suppress_build_script_link_lines=false
             pointer_width=64"#]];
         expected.assert_eq(&config_file);
@@ -736,6 +844,7 @@ mod test {
             &Target::from_target_triple(Some("x86_64-unknown-linux-musl".to_string())).unwrap(),
             InterpreterKind::CPython,
             (3, 11),
+            "",
         )
         .unwrap();
         assert_eq!(sysconfig.ext_suffix, ".cpython-311-x86_64-linux-musl.so");
@@ -745,7 +854,7 @@ mod test {
             version=3.11
             shared=true
             abi3=false
-            build_flags=WITH_THREAD
+            build_flags=
             suppress_build_script_link_lines=false
             pointer_width=64"#]];
         expected.assert_eq(&config_file);
