@@ -35,6 +35,7 @@ pub enum Os {
     Emscripten,
     Wasi,
     Aix,
+    Hurd,
 }
 
 impl fmt::Display for Os {
@@ -54,6 +55,7 @@ impl fmt::Display for Os {
             Os::Emscripten => write!(f, "Emscripten"),
             Os::Wasi => write!(f, "Wasi"),
             Os::Aix => write!(f, "AIX"),
+            Os::Hurd => write!(f, "Hurd"),
         }
     }
 }
@@ -63,6 +65,7 @@ impl fmt::Display for Os {
 #[serde(rename_all = "lowercase")]
 pub enum Arch {
     Aarch64,
+    Armv5teL,
     Armv6L,
     Armv7L,
     #[serde(alias = "ppc")]
@@ -83,6 +86,7 @@ pub enum Arch {
     Mipsel,
     Mips,
     Sparc64,
+    Sparcv9,
     LoongArch64,
 }
 
@@ -90,6 +94,7 @@ impl fmt::Display for Arch {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
             Arch::Aarch64 => write!(f, "aarch64"),
+            Arch::Armv5teL => write!(f, "armv5tel"),
             Arch::Armv6L => write!(f, "armv6l"),
             Arch::Armv7L => write!(f, "armv7l"),
             Arch::Powerpc => write!(f, "ppc"),
@@ -106,6 +111,7 @@ impl fmt::Display for Arch {
             Arch::Mipsel => write!(f, "mipsel"),
             Arch::Mips => write!(f, "mips"),
             Arch::Sparc64 => write!(f, "sparc64"),
+            Arch::Sparcv9 => write!(f, "sparcv9"),
             Arch::LoongArch64 => write!(f, "loongarch64"),
         }
     }
@@ -120,7 +126,7 @@ impl Arch {
         // MACHINE_ARCH	vs MACHINE_CPUARCH vs MACHINE section
         match self {
             Arch::Aarch64 => "arm64",
-            Arch::Armv6L | Arch::Armv7L => "arm",
+            Arch::Armv5teL | Arch::Armv6L | Arch::Armv7L => "arm",
             Arch::Powerpc | Arch::Powerpc64Le | Arch::Powerpc64 => "powerpc",
             Arch::X86 => "i386",
             Arch::X86_64 => "amd64",
@@ -128,6 +134,7 @@ impl Arch {
             Arch::Mips64el | Arch::Mips64 | Arch::Mipsel | Arch::Mips => "mips",
             // sparc64 is unsupported since FreeBSD 13.0
             Arch::Sparc64 => "sparc64",
+            Arch::Sparcv9 => "sparcv9",
             Arch::Wasm32 => "wasm32",
             Arch::S390X => "s390x",
             Arch::LoongArch64 => "loongarch64",
@@ -140,6 +147,7 @@ fn get_supported_architectures(os: &Os) -> Vec<Arch> {
     match os {
         Os::Linux => vec![
             Arch::Aarch64,
+            Arch::Armv5teL,
             Arch::Armv6L,
             Arch::Armv7L,
             Arch::Powerpc,
@@ -190,9 +198,10 @@ fn get_supported_architectures(os: &Os) -> Vec<Arch> {
         Os::Dragonfly => vec![Arch::X86_64],
         Os::Illumos => vec![Arch::X86_64],
         Os::Haiku => vec![Arch::X86_64],
-        Os::Solaris => vec![Arch::X86_64, Arch::Sparc64],
+        Os::Solaris => vec![Arch::X86_64, Arch::Sparc64, Arch::Sparcv9],
         Os::Emscripten | Os::Wasi => vec![Arch::Wasm32],
         Os::Aix => vec![Arch::Powerpc64],
+        Os::Hurd => vec![Arch::X86, Arch::X86_64],
     }
 }
 
@@ -243,8 +252,8 @@ impl Target {
         let os = match platform.operating_system {
             OperatingSystem::Linux => Os::Linux,
             OperatingSystem::Windows => Os::Windows,
-            OperatingSystem::MacOSX { .. } | OperatingSystem::Darwin => Os::Macos,
-            OperatingSystem::Ios => Os::Ios,
+            OperatingSystem::MacOSX(_) | OperatingSystem::Darwin(_) => Os::Macos,
+            OperatingSystem::IOS(_) => Os::Ios,
             OperatingSystem::Netbsd => Os::NetBsd,
             OperatingSystem::Freebsd => Os::FreeBsd,
             OperatingSystem::Openbsd => Os::OpenBsd,
@@ -255,6 +264,7 @@ impl Target {
             OperatingSystem::Emscripten => Os::Emscripten,
             OperatingSystem::Wasi | OperatingSystem::WasiP1 | OperatingSystem::WasiP2 => Os::Wasi,
             OperatingSystem::Aix => Os::Aix,
+            OperatingSystem::Hurd => Os::Hurd,
             unsupported => bail!("The operating system {:?} is not supported", unsupported),
         };
 
@@ -262,8 +272,10 @@ impl Target {
             Architecture::X86_64 | Architecture::X86_64h => Arch::X86_64,
             Architecture::X86_32(_) => Arch::X86,
             Architecture::Arm(arm_arch) => match arm_arch {
+                ArmArchitecture::Armv5te => Arch::Armv5teL,
                 ArmArchitecture::Arm | ArmArchitecture::Armv6 => Arch::Armv6L,
-                _ => Arch::Armv7L,
+                ArmArchitecture::Armv7 => Arch::Armv7L,
+                _ => bail!("The architecture {} is not supported", arm_arch),
             },
             Architecture::Aarch64(_) => Arch::Aarch64,
             Architecture::Powerpc => Arch::Powerpc,
@@ -284,6 +296,7 @@ impl Target {
                 _ => bail!("The architecture {} is not supported", mips32_arch),
             },
             Architecture::Sparc64 => Arch::Sparc64,
+            Architecture::Sparcv9 => Arch::Sparcv9,
             Architecture::LoongArch64 => Arch::LoongArch64,
             unsupported => bail!("The architecture {} is not supported", unsupported),
         };
@@ -351,6 +364,7 @@ impl Target {
     pub fn get_python_arch(&self) -> &str {
         match self.arch {
             Arch::Aarch64 => "aarch64",
+            Arch::Armv5teL => "armv5tel",
             Arch::Armv6L => "armv6l",
             Arch::Armv7L => "armv7l",
             Arch::Powerpc => "ppc",
@@ -366,13 +380,17 @@ impl Target {
             Arch::Mips64el | Arch::Mips64 => "mips64",
             Arch::Mipsel | Arch::Mips => "mips",
             Arch::Sparc64 => "sparc64",
+            Arch::Sparcv9 => "sparcv9",
             Arch::LoongArch64 => "loongarch64",
         }
     }
 
     /// Returns the extension architecture name python uses in `ext_suffix` for this architecture.
     pub fn get_python_ext_arch(&self, python_impl: InterpreterKind) -> &str {
-        if matches!(self.target_arch(), Arch::Armv6L | Arch::Armv7L) {
+        if matches!(
+            self.target_arch(),
+            Arch::Armv5teL | Arch::Armv6L | Arch::Armv7L
+        ) {
             "arm"
         } else if matches!(self.target_arch(), Arch::Powerpc64Le)
             && python_impl == InterpreterKind::PyPy
@@ -426,6 +444,7 @@ impl Target {
             // This isn't real, there's no sys.platform here
             Os::Wasi => "wasi",
             Os::Aix => "aix",
+            Os::Hurd => "gnu",
         }
     }
 
@@ -444,17 +463,19 @@ impl Target {
                     PlatformTag::manylinux2010()
                 }
             }
-            Arch::Armv6L
+            Arch::Riscv64 => PlatformTag::Manylinux { x: 2, y: 31 },
+            Arch::LoongArch64 => PlatformTag::Manylinux { x: 2, y: 36 },
+            Arch::Armv5teL
+            | Arch::Armv6L
             | Arch::Wasm32
             | Arch::Riscv32
-            | Arch::Riscv64
             | Arch::Mips64el
             | Arch::Mips64
             | Arch::Mipsel
             | Arch::Mips
             | Arch::Powerpc
             | Arch::Sparc64
-            | Arch::LoongArch64 => PlatformTag::Linux,
+            | Arch::Sparcv9 => PlatformTag::Linux,
         }
     }
 
@@ -470,8 +491,10 @@ impl Target {
             | Arch::Mips64el
             | Arch::Mips64
             | Arch::Sparc64
+            | Arch::Sparcv9
             | Arch::LoongArch64 => 64,
-            Arch::Armv6L
+            Arch::Armv5teL
+            | Arch::Armv6L
             | Arch::Armv7L
             | Arch::X86
             | Arch::Wasm32
@@ -510,7 +533,8 @@ impl Target {
             | Os::Haiku
             | Os::Emscripten
             | Os::Wasi
-            | Os::Aix => true,
+            | Os::Aix
+            | Os::Hurd => true,
         }
     }
 
@@ -586,6 +610,18 @@ impl Target {
         self.os == Os::Wasi
     }
 
+    /// Returns true if we're building a binary for GNU/Hurd
+    #[inline]
+    pub fn is_hurd(&self) -> bool {
+        self.os == Os::Hurd
+    }
+
+    /// Returns true if we're building a binary for AIX
+    #[inline]
+    pub fn is_aix(&self) -> bool {
+        self.os == Os::Aix
+    }
+
     /// Returns true if the current platform's target env is Musl
     #[inline]
     pub fn is_musl_libc(&self) -> bool {
