@@ -9,6 +9,7 @@ use maturin::pyproject_toml::SdistGenerator;
 use rstest::rstest;
 use std::env;
 use std::path::Path;
+use std::time::Duration;
 use time::macros::datetime;
 use which::which;
 
@@ -120,8 +121,8 @@ fn develop_pyo3_mixed_src_layout() {
 #[test]
 fn develop_cffi_pure() {
     let python_implementation = test_python_implementation().unwrap();
-    if cfg!(windows) && env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
-        // PyPy on Windows hangs on cffi test sometimes
+    if env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
+        // TODO: PyPy hangs on cffi test sometimes
         return;
     }
     handle_result(develop::test_develop(
@@ -136,8 +137,8 @@ fn develop_cffi_pure() {
 #[test]
 fn develop_cffi_mixed() {
     let python_implementation = test_python_implementation().unwrap();
-    if cfg!(windows) && env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
-        // PyPy on Windows hangs on cffi test sometimes
+    if env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
+        // PyPy hangs on cffi test sometimes
         return;
     }
     handle_result(develop::test_develop(
@@ -200,6 +201,7 @@ fn develop_uniffi_multiple_binding_files() {
 }
 
 #[rstest]
+#[timeout(Duration::from_secs(60))]
 #[case(TestInstallBackend::Pip, "pip")]
 #[case(TestInstallBackend::Uv, "uv")]
 #[test]
@@ -226,6 +228,7 @@ fn develop_hello_world(#[case] backend: TestInstallBackend, #[case] name: &str)
 }
 
 #[rstest]
+#[timeout(Duration::from_secs(60))]
 #[case(TestInstallBackend::Pip, "pip")]
 #[case(TestInstallBackend::Uv, "uv")]
 #[test]
@@ -371,8 +374,8 @@ fn integration_pyo3_pure_conda() {
 #[test]
 fn integration_cffi_pure() {
     let python_implementation = test_python_implementation().unwrap();
-    if cfg!(windows) && env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
-        // PyPy on Windows hangs on cffi test sometimes
+    if env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
+        // PyPy hangs on cffi test sometimes
         return;
     }
     handle_result(integration::test_integration(
@@ -387,8 +390,8 @@ fn integration_cffi_pure() {
 #[test]
 fn integration_cffi_mixed() {
     let python_implementation = test_python_implementation().unwrap();
-    if cfg!(windows) && env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
-        // PyPy on Windows hangs on cffi test sometimes
+    if env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
+        // PyPy hangs on cffi test sometimes
         return;
     }
     handle_result(integration::test_integration(
@@ -481,6 +484,17 @@ fn integration_readme_duplication() {
     ));
 }
 
+#[test]
+fn integration_workspace_inverted_order() {
+    handle_result(integration::test_integration(
+        "test-crates/workspace-inverted-order/path-dep-with-root",
+        None,
+        "integration-workspace-inverted-order",
+        false,
+        None,
+    ));
+}
+
 #[test]
 // Sourced from https://pypi.org/project/wasmtime/11.0.0/#files
 // update with wasmtime updates
@@ -504,11 +518,12 @@ fn integration_wasm_hello_world() {
         None,
         "integration-wasm-hello-world",
         false,
-        Some("wasm32-wasi"),
+        Some("wasm32-wasip1"),
     ));
 
     let python_implementation = test_python_implementation().unwrap();
-    let venv_name = format!("integration-wasm-hello-world-py3-wasm32-wasi-{python_implementation}");
+    let venv_name =
+        format!("integration-wasm-hello-world-py3-wasm32-wasip1-{python_implementation}");
 
     // Make sure we're actually running wasm
     assert!(Path::new("test-crates")
@@ -628,12 +643,16 @@ fn workspace_members_non_local_dep_sdist() {
         license = "MIT"
 
         [dependencies]
-        pyo3 = { version = "0.22.0", features = ["abi3-py37", "extension-module", "generate-import-lib"] }
+        pyo3 = { version = "0.24.0", features = [
+            "abi3-py37",
+            "extension-module",
+            "generate-import-lib",
+        ] }
 
         [lib]
         name = "pyo3_pure"
         crate-type = ["cdylib"]
-        "#]];
+    "#]];
     handle_result(other::test_source_distribution(
         "test-crates/pyo3-pure",
         SdistGenerator::Cargo,
@@ -693,8 +712,7 @@ fn lib_with_target_path_dep_sdist() {
         crate-type = ["cdylib"]
 
         [dependencies]
-        # Don't use the macros feature, which makes compilation much faster
-        pyo3 = { version = "0.22.0", default-features = false, features = ["extension-module"] }
+        pyo3 = { version = "0.24.0", features = ["extension-module"] }
 
         [target.'cfg(not(target_endian = "all-over-the-place"))'.dependencies]
         some_path_dep = { path = "../some_path_dep" }
